"use strict";
/**
 * 	All File System operations are maint to be in this file
 *  This means that no import of 'fs' module shouls apear elsewere
 *
 *  We also tries to use sync functions in order not to publish async functions
 *  for those basic stiff. It could be changed at any moment
 *  by using fs.promises...
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.$copyFile = exports.$realMoveFile = exports.$removeFile = exports.$localRenameFile = exports.$fullWriteBuffer = exports.$writeBuffer = exports.$readData = exports.$readBuffer = exports.$fullWriteString = exports.$writeString = exports.$readString = exports.$loadJSON = exports.$filename = exports.$dir = exports.$newext = exports.$withoutext = exports.$ext = exports.$path = exports.$normalizepath = exports.$isabsolutepath = exports.$uniquefile = exports.$temporarypath = exports.$filesize = exports.$createDirectory = exports.$isdirectory = exports.$iswritable = exports.$chmod = exports.$isreadable = exports.$isexecutable = exports.$isfile = exports.$stats = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const commons_1 = require("./commons");
const tsdefaults_1 = require("./tsdefaults");
const crypto_1 = require("./crypto");
const utils_1 = require("./utils");
const tsdata_1 = require("./tsdata");
const tserrors_1 = require("./tserrors");
// if $stats() returns null it means that the path does not exist.
function $stats(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $stats() function in browser';
    }
    if ((0, commons_1.$length)(src)) {
        try {
            return (0, fs_1.statSync)(src);
        }
        catch (e) {
            if ((e === null || e === void 0 ? void 0 : e.code) === 'ENOENT')
                return null;
            throw e;
        }
    }
    return null;
}
exports.$stats = $stats;
function $isfile(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $isfile() function in browser';
    }
    const stats = $stats(src);
    return (0, commons_1.$ok)(stats) ? stats.isFile() : false;
}
exports.$isfile = $isfile;
function $isexecutable(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $isexecutable() function in browser';
    }
    return $isfile(src) && _safeCheckPermissions(src, fs_1.constants.R_OK | fs_1.constants.X_OK);
}
exports.$isexecutable = $isexecutable;
function $isreadable(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $isreadable() function in browser';
    }
    return (0, commons_1.$ok)($stats(src)) && _safeCheckPermissions(src, fs_1.constants.R_OK);
}
exports.$isreadable = $isreadable;
function $chmod(src, mode) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $chmod() function in browser';
    }
    let ret = false;
    if ((0, commons_1.$length)(src) && (0, commons_1.$isunsigned)(mode) && mode <= 0o777) {
        try {
            (0, fs_1.chmodSync)(src, mode);
            ret = true;
        }
        catch (_a) {
            ret = false;
        }
    }
    return ret;
}
exports.$chmod = $chmod;
function $iswritable(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $iswritable() function in browser';
    }
    return (0, commons_1.$ok)($stats(src)) && _safeCheckPermissions(src, fs_1.constants.W_OK);
}
exports.$iswritable = $iswritable;
function $isdirectory(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $isdirectory() function in browser';
    }
    const stats = $stats(src);
    return (0, commons_1.$ok)(stats) ? stats.isDirectory() : false;
}
exports.$isdirectory = $isdirectory;
function $createDirectory(p) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $createDirectory() function in browser';
    }
    let ret = false;
    if ((0, commons_1.$length)(p)) {
        const stats = $stats(p);
        if ((0, commons_1.$ok)(stats) && stats.isDirectory()) {
            ret = true;
        }
        else if (!(0, commons_1.$ok)(stats) || !stats.isFile()) {
            try {
                (0, fs_1.mkdirSync)(p, { recursive: true });
                ret = true;
            }
            catch (_a) {
                ret = false;
            }
        }
    }
    return ret;
}
exports.$createDirectory = $createDirectory;
function $filesize(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $filesize() function in browser';
    }
    const stats = $stats(src);
    return (0, commons_1.$ok)(stats) ? stats.size : 0;
}
exports.$filesize = $filesize;
function $temporarypath(ext = '', src = '') {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $temporarypath() function in browser';
    }
    ext = (0, commons_1.$ftrim)(ext);
    let file = $uniquefile(src);
    if ((0, commons_1.$length)(ext)) {
        file = $newext(file, ext);
    }
    return $path((0, tsdefaults_1.$tmp)(), file);
}
exports.$temporarypath = $temporarypath;
function $uniquefile(src, e = undefined, internalImplementation = false) {
    const rand = (0, crypto_1.$uuid)();
    e = (0, commons_1.$ftrim)(e);
    if (!(0, commons_1.$length)(src)) {
        return $newext(rand, e, internalImplementation);
    }
    const finalExt = e.length ? e : $ext(src);
    return $newext(`${$withoutext(src, internalImplementation)}-${rand}`, finalExt, internalImplementation);
}
exports.$uniquefile = $uniquefile;
function $isabsolutepath(src, internalImplementation = false) {
    return (0, commons_1.$length)(src) ? (internalImplementation || (0, utils_1.$inbrowser)() ? src.startsWith('/') : (0, path_1.isAbsolute)(src)) : false;
}
exports.$isabsolutepath = $isabsolutepath;
function $normalizepath(src, internalImplementation = false) {
    if (!(0, commons_1.$length)(src)) {
        return '';
    }
    return internalImplementation || (0, utils_1.$inbrowser)() ? $path(true, src) : (0, path_1.normalize)(src);
}
exports.$normalizepath = $normalizepath;
function $path(first, ...paths) {
    function _internalPath(...paths) {
        if (paths.length > 0) {
            const isAbsolute = $isabsolutepath(paths[0], true);
            let comps = [];
            for (let p of paths) {
                if (p.length > 0) {
                    for (let s of p.split('/')) {
                        if (s === '..') {
                            if (comps.length > 0) {
                                if (comps[comps.length - 1] != '..') {
                                    comps.pop();
                                }
                                else {
                                    comps.push('..');
                                }
                            }
                            else if (!isAbsolute) {
                                comps.push('..');
                            }
                        }
                        else if (s.length > 0 && s !== '.') {
                            comps.push(s);
                        }
                    }
                }
            }
            if (comps.length) {
                const s = comps.join('/');
                return isAbsolute ? '/' + s : s;
            }
            else if (isAbsolute) {
                return '/';
            }
        }
        return '';
    }
    if (!(0, commons_1.$isstring)(first)) {
        return first || (0, utils_1.$inbrowser)() ? _internalPath(...paths) : (0, path_1.join)(...paths);
    }
    return (0, utils_1.$inbrowser)() ? _internalPath(first, ...paths) : (0, path_1.join)(first, ...paths);
}
exports.$path = $path;
function $ext(s, internalImplementation = false) {
    if ((0, commons_1.$length)(s)) {
        if (internalImplementation || (0, utils_1.$inbrowser)()) {
            const p = s.lastIndexOf('.');
            return p >= 0 ? s.slice(p + 1) : '';
        }
        else {
            const e = (0, path_1.extname)(s);
            if ((0, commons_1.$length)(e)) {
                return e.slice(1);
            }
        }
    }
    return '';
}
exports.$ext = $ext;
function $withoutext(s, internalImplementation) {
    if (!(0, commons_1.$length)(s)) {
        return '';
    }
    const e = $ext(s, internalImplementation);
    s = e.length ? s.slice(0, s.length - e.length) : s;
    return s.length && s.charAt(s.length - 1) === '.' ? s.slice(0, s.length - 1) : s;
}
exports.$withoutext = $withoutext;
function $newext(s, e = undefined, internalImplementation) {
    let b = $withoutext(s, internalImplementation);
    return (0, commons_1.$length)(e) ? `${b}.${e}` : b;
}
exports.$newext = $newext;
function $dir(s, internalImplementation = false) {
    if (!(0, commons_1.$length)(s)) {
        return '';
    }
    if (internalImplementation || (0, utils_1.$inbrowser)()) {
        const p = s.lastIndexOf('/');
        return p === 0 ? '/' : (p > 0 ? s.slice(0, p) : '.');
    }
    return (0, path_1.dirname)(s);
}
exports.$dir = $dir;
function $filename(s, internalImplementation = false) {
    if (!(0, commons_1.$length)(s)) {
        return '';
    }
    if (internalImplementation || (0, utils_1.$inbrowser)()) {
        const p = s.lastIndexOf('/');
        return p >= 0 ? s.slice(p + 1) : s;
    }
    return (0, path_1.basename)(s);
}
exports.$filename = $filename;
function $loadJSON(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable function in browser';
    }
    let ret = null;
    if ((0, commons_1.$length)(src)) {
        let loadedString = src instanceof Buffer ? src.toString('utf-8') : $readString(src, 'utf-8');
        if ((0, commons_1.$length)(loadedString)) {
            try {
                ret = JSON.parse(loadedString);
                ret = (0, commons_1.$ok)(ret) ? ret : null;
            }
            catch (e) {
                console.log(`Impossible to parse JSON file ${src}`);
                ret = null;
            }
        }
    }
    return ret;
}
exports.$loadJSON = $loadJSON;
function $readString(src, encoding = 'utf-8') {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $readString() function in browser';
    }
    let ret = null;
    if ((0, commons_1.$length)(src) && Buffer.isEncoding(encoding)) {
        try {
            ret = (0, fs_1.readFileSync)(src, encoding);
        }
        catch (e) {
            ret = null;
        }
    }
    return ret;
}
exports.$readString = $readString;
function $writeString(src, str, opts = {}) {
    const [res,] = $fullWriteString(src, str, opts);
    return res;
}
exports.$writeString = $writeString;
// returns [OK/KO and the name of the precedent version if it exists
function $fullWriteString(src, str, opts) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $writeString() function in browser';
    }
    let encoding = (0, commons_1.$length)(opts.encoding) ? opts.encoding : 'utf-8';
    if (!Buffer.isEncoding(encoding)) {
        return [false, null];
    }
    return $fullWriteBuffer(src, Buffer.from(str, encoding), opts);
}
exports.$fullWriteString = $fullWriteString;
function $readBuffer(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $readBuffer() function in browser';
    }
    let ret = null;
    if ((0, commons_1.$length)(src)) {
        try {
            ret = (0, fs_1.readFileSync)(src);
        } // readFile without any encoding option returns a buffer
        catch (e) {
            ret = null;
        }
    }
    return ret;
}
exports.$readBuffer = $readBuffer;
function $readData(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $readData() function in browser';
    }
    const buf = $readBuffer(src);
    return (0, commons_1.$ok)(buf) ? new tsdata_1.TSData(buf, { dontCopySourceBuffer: true }) : null;
}
exports.$readData = $readData;
function $writeBuffer(src, buf, opts = {}) {
    const [res,] = $fullWriteBuffer(src, buf, opts);
    return res;
}
exports.$writeBuffer = $writeBuffer;
// returns [OK/KO and the name of the precedent version if it exists
function $fullWriteBuffer(src, buf, opts) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $writeBuffer() function in browser';
    }
    let done = false;
    let precedent = null;
    let start = (0, commons_1.$ok)(opts.byteStart) ? opts.byteStart : 0;
    let end = (0, commons_1.$ok)(opts.byteEnd) ? opts.byteEnd : buf.byteLength;
    if ((0, commons_1.$length)(src) && (0, commons_1.$isunsigned)(start) || !(0, commons_1.$isunsigned)(end)) {
        const pathToWrite = opts.attomically ? $uniquefile(src) : src;
        end = Math.min(end, buf.byteLength);
        start = Math.min(start, end);
        // never move the next line of code before this point because 
        // TSData.byteLength may be different from its internal storage buffer length
        // warning: this method works because it's not async so we can
        // consider that TSData is immutable during this scope
        if (buf instanceof tsdata_1.TSData) {
            [buf,] = buf.internalStorage;
        }
        try {
            const fd = (0, fs_1.openSync)(pathToWrite, 'w', 0o666);
            const MAX_TRY = 3;
            try {
                let retries = 0;
                while (start < end) {
                    const wlen = (0, fs_1.writeSync)(fd, buf, start, end - start);
                    if (wlen === 0) {
                        retries++;
                    }
                    else {
                        retries = 0;
                    } // we never should have wlen === 0 but ...
                    if (retries > MAX_TRY) {
                        throw `Tried to fs.writeSync() ${retries} times without any success.`;
                    }
                    start += wlen;
                }
                done = true;
            }
            finally {
                (0, fs_1.closeSync)(fd);
            }
        }
        catch (e) {
            done = false;
        }
        if (done && opts.attomically) {
            const renamedExistingFile = $uniquefile(src);
            done = _safeRename(src, renamedExistingFile);
            if (done && !_safeRename(pathToWrite, src)) {
                // we immediately try to give back its name to our original file
                if (!_safeRename(renamedExistingFile, src)) {
                    // we should have been able to give our initial file its original name back
                    // but we could'nt do it, so, in this very hypothetical case, we will not
                    // destroy anything and will throw an Error will all the infos in it 
                    throw new tserrors_1.TSError(`Unable to atomically finish writing file '${src}'`, {
                        wantedPath: src,
                        renamedExistingFile: renamedExistingFile,
                        writtenDataFile: pathToWrite
                    });
                }
                done = false;
            }
            if (!done) {
                _safeUnlink(pathToWrite);
            } // we may let a newly created temporary file here if unlink does not succeed
            else if (opts.removePrecedentVersion) {
                _safeUnlink(renamedExistingFile);
            }
            else {
                precedent = renamedExistingFile;
            }
        }
    }
    return [done, precedent];
}
exports.$fullWriteBuffer = $fullWriteBuffer;
function $localRenameFile(src, dest) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $localRenameFile() function in browser';
    }
    return (0, commons_1.$length)(src) > 0 && (0, commons_1.$length)(dest) > 0 && src !== dest && $isfile(src) && _safeRename(src, dest);
}
exports.$localRenameFile = $localRenameFile;
function $removeFile(src) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $removeFile() function in browser';
    }
    return (0, commons_1.$length)(src) > 0 && $isfile(src) && _safeUnlink(src);
}
exports.$removeFile = $removeFile;
/*
    src is a file
    dest is a file or a directory
    if it's a directory, it must exist and the src filename is used
 */
function $realMoveFile(src, dest) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $realMoveFile() function in browser';
    }
    let done = false;
    if ((0, commons_1.$length)(src) && (0, commons_1.$length)(dest) && src !== dest && $isfile(src)) {
        if ($isdirectory(dest)) {
            dest = $path(dest, $filename(src));
            if (src === dest) {
                return false;
            }
        }
        done = _safeRename(src, dest);
        if (!done) {
            // rename function did not work, so we will try to copy
            // the file
            try {
                (0, fs_1.copyFileSync)(src, dest);
                (0, fs_1.unlinkSync)(src);
                done = true;
            }
            catch (_a) {
                done = false;
            } // WARNING: if we fails here, we may have made the copy() but not the unlink()
        }
    }
    return done;
}
exports.$realMoveFile = $realMoveFile;
function $copyFile(src, dest, overwrite = false) {
    if ((0, utils_1.$inbrowser)()) {
        throw 'unavailable $copyFile() function in browser';
    }
    let done = false;
    if ((0, commons_1.$length)(src) && (0, commons_1.$length)(dest) && src !== dest && $isfile(src)) {
        if ($isdirectory(dest)) {
            dest = $path(dest, $filename(src));
        }
        if (src === dest || $isdirectory(dest) || (!overwrite && $isfile(dest))) {
            return false;
        }
        try {
            (0, fs_1.copyFileSync)(src, dest, (overwrite ? 0 : fs_1.constants.COPYFILE_EXCL));
            done = true;
        }
        catch (_a) {
            done = false;
        }
    }
    return done;
}
exports.$copyFile = $copyFile;
function _safeRename(s, d) { let done = false; try {
    (0, fs_1.renameSync)(s, d);
    done = true;
}
catch (_a) {
    done = false;
} return done; }
function _safeUnlink(p) { let done = false; try {
    (0, fs_1.unlinkSync)(p);
    done = true;
}
catch (_a) {
    done = false;
} return done; }
function _safeCheckPermissions(src, permissions) {
    let ret = false;
    try {
        if ((0, fs_1.statSync)(src).isFile()) {
            // we cound use stats.mode but we should take care of who we are, so better to rely on accessSync(-)
            (0, fs_1.accessSync)(src, permissions);
            ret = true;
        }
    }
    catch (_a) {
        ret = false;
    }
    return ret;
}
//# sourceMappingURL=fs.js.map