"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSDataCursor = void 0;
const os_1 = require("os");
const commons_1 = require("./commons");
const tsdata_1 = require("./tsdata");
class TSDataCursor extends tsdata_1.TSData {
    constructor(source, opts = {}) {
        super(source, {
            dontCopySourceBuffer: opts.dontCopySourceBuffer,
            fillWithZeros: opts.fillWithZeros,
            allocMethod: opts.allocMethod
        });
        this._cursor = 0;
        if ((0, commons_1.$ok)(opts.cursor)) {
            this.cursor = opts.cursor;
        }
        this._endianness = (0, commons_1.$ok)(opts.endianness) ? opts.endianness : (0, os_1.endianness)();
    }
    get bigEndian() { return this._endianness === 'BE'; }
    get littleEndian() { return this._endianness === 'LE'; }
    get endianness() { return this._endianness; }
    get cursor() { return this._cursor; }
    set cursor(n) {
        if (!(0, commons_1.$isunsigned)(n)) {
            throw `TSData.cursor = ${n} is not valid.`;
        }
        if (n > this._len) {
            this.length = n;
        }
        this._cursor = n;
    }
    readBigUInt64(offset) { return this._read(offset, 8, this.bigEndian ? Buffer.prototype.readBigUInt64BE : Buffer.prototype.readBigUInt64LE); }
    readBigInt64(offset) { return this._read(offset, 8, this.bigEndian ? Buffer.prototype.readBigInt64BE : Buffer.prototype.readBigInt64LE); }
    readUInt16(offset) { return this._read(offset, 2, this.bigEndian ? Buffer.prototype.readUInt16BE : Buffer.prototype.readUInt16LE); }
    readUInt32(offset) { return this._read(offset, 4, this.bigEndian ? Buffer.prototype.readUInt32BE : Buffer.prototype.readUInt32LE); }
    readInt16(offset) { return this._read(offset, 2, this.bigEndian ? Buffer.prototype.readInt16BE : Buffer.prototype.readInt16LE); }
    readInt32(offset) { return this._read(offset, 4, this.bigEndian ? Buffer.prototype.readInt32BE : Buffer.prototype.readInt32LE); }
    readFloat(offset) { return this._read(offset, 4, this.bigEndian ? Buffer.prototype.readFloatBE : Buffer.prototype.readFloatLE); }
    readDouble(offset) { return this._read(offset, 8, this.bigEndian ? Buffer.prototype.readDoubleBE : Buffer.prototype.readDoubleLE); }
    writeBigInt64(value, offset) { this._write(value, offset, 8, this.bigEndian ? Buffer.prototype.writeBigInt64BE : Buffer.prototype.writeBigInt64LE); }
    writeBigUInt64(value, offset) { this._write(value, offset, 8, this.bigEndian ? Buffer.prototype.writeBigUInt64BE : Buffer.prototype.writeBigUInt64LE); }
    writeUInt16(value, offset) { this._write(value, offset, 2, this.bigEndian ? Buffer.prototype.writeUInt16BE : Buffer.prototype.writeUInt16LE); }
    writeUInt32(value, offset) { this._write(value, offset, 4, this.bigEndian ? Buffer.prototype.writeUInt32BE : Buffer.prototype.writeUInt32LE); }
    writeInt16(value, offset) { this._write(value, offset, 2, this.bigEndian ? Buffer.prototype.writeInt16BE : Buffer.prototype.writeInt16LE); }
    writeInt32(value, offset) { this._write(value, offset, 4, this.bigEndian ? Buffer.prototype.writeInt32BE : Buffer.prototype.writeInt32LE); }
    writeFloat(value, offset) { this._write(value, offset, 4, this.bigEndian ? Buffer.prototype.writeFloatBE : Buffer.prototype.writeFloatLE); }
    writeDouble(value, offset) { this._write(value, offset, 8, this.bigEndian ? Buffer.prototype.writeDoubleBE : Buffer.prototype.writeDoubleLE); }
    _read(offset = this._cursor, size, bufferReadFn) {
        const v = super._read(offset, size, bufferReadFn);
        this._cursor = offset + size;
        return v;
    }
    _write(value, offset = this._cursor, size, bufferWriteFn) {
        super._write(value, offset, size, bufferWriteFn);
        this._cursor = offset + size;
    }
}
exports.TSDataCursor = TSDataCursor;
//# sourceMappingURL=tsdatacursor.js.map