"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$durationNumber2StringFormat = exports.$duration2String = exports.$duration = exports.$durationcomponents = exports.$components2stringformat = exports.$components2StringWithOffset = exports.$components2string = exports.$components2date = exports.$components2timestamp = exports.$isostring2components = exports.$parsedate = exports.$parsedatetime = exports.$parsetime = exports.$componentshavetime = exports.$componentsarevalid = exports.$components = exports.$timecomponents = exports.TSDateRep = exports.TSDateForm = void 0;
const commons_1 = require("./commons");
const tsdefaults_1 = require("./tsdefaults");
const tsdate_1 = require("./tsdate");
const types_1 = require("./types");
const tscountry_1 = require("./tscountry");
var TSDateForm;
(function (TSDateForm) {
    TSDateForm[TSDateForm["Standard"] = 0] = "Standard";
    TSDateForm[TSDateForm["English"] = 1] = "English";
    TSDateForm[TSDateForm["Computer"] = 2] = "Computer";
    TSDateForm[TSDateForm["ISO8601"] = 3] = "ISO8601";
    TSDateForm[TSDateForm["ISO8601C"] = 4] = "ISO8601C";
    TSDateForm[TSDateForm["ISO8601L"] = 5] = "ISO8601L";
})(TSDateForm = exports.TSDateForm || (exports.TSDateForm = {}));
var TSDateRep;
(function (TSDateRep) {
    TSDateRep["LocalTime"] = "lt";
    TSDateRep["LocalDate"] = "ld";
    TSDateRep["LocalDateTime"] = "ldt";
    TSDateRep["ShortLocalDate"] = "sld";
    TSDateRep["ShortLocalDateTime"] = "sldt";
})(TSDateRep = exports.TSDateRep || (exports.TSDateRep = {}));
/**
 * If you call timecomponents() with no parameters, it
 * returns the components for the current local time.
 * This function can not have a string as parameter ;
 * in order to create components from string, you need
 * to user functions $parsetime()
 */
function $timecomponents(source) {
    if (!(0, commons_1.$ok)(source)) {
        source = new Date();
    }
    else if (source instanceof tsdate_1.TSDate) {
        source = source.timestamp;
    }
    if ((0, commons_1.$isnumber)(source)) {
        const timestamp = source;
        return {
            hour: (0, tsdate_1.$hourFromTimestamp)(timestamp),
            minute: (0, tsdate_1.$minuteFromTimestamp)(timestamp),
            second: (0, tsdate_1.$secondFromTimestamp)(timestamp),
        };
    }
    const d = source;
    return {
        hour: d.getHours(),
        minute: d.getMinutes(),
        second: d.getSeconds(),
    };
}
exports.$timecomponents = $timecomponents;
/**
 * If you call components() with no parameters, it
 * returns the components for the current local date.
 * This function can not have a string as parameter ;
 * in order to create components from string, you need
 * to user functions $parsedate(), $parsedatetime() or $isostring2components()
 */
function $components(source) {
    if (!(0, commons_1.$ok)(source)) {
        source = new Date();
    }
    else if (source instanceof tsdate_1.TSDate) {
        source = source.timestamp;
    }
    if ((0, commons_1.$isnumber)(source)) {
        const timestamp = source;
        let Z = Math.floor(timestamp / tsdate_1.TSDay) + tsdate_1.TSDaysFrom00000229To20010101;
        let gg = Z - 0.25;
        let CENTURY = Math.floor(gg / 36524.25);
        let CENTURY_MQUART = CENTURY - Math.floor(CENTURY / 4);
        let ALLDAYS = gg + CENTURY_MQUART;
        let Y = Math.floor(ALLDAYS / 365.25);
        let Y365 = Math.floor(Y * 365.25);
        let DAYS_IN_Y = CENTURY_MQUART + Z - Y365;
        let MONTH_IN_Y = Math.floor((5 * DAYS_IN_Y + 456) / 153);
        return {
            day: Math.floor(DAYS_IN_Y - Math.floor((153 * MONTH_IN_Y - 457) / 5)),
            hour: (0, tsdate_1.$hourFromTimestamp)(timestamp),
            minute: (0, tsdate_1.$minuteFromTimestamp)(timestamp),
            second: (0, tsdate_1.$secondFromTimestamp)(timestamp),
            dayOfWeek: ((Z + 2) % 7),
            month: (MONTH_IN_Y > 12 ? MONTH_IN_Y - 12 : MONTH_IN_Y),
            year: (MONTH_IN_Y > 12 ? Y + 1 : Y)
        };
    }
    const d = source;
    return {
        year: d.getFullYear(),
        month: (d.getMonth() + 1),
        day: d.getDate(),
        hour: d.getHours(),
        minute: d.getMinutes(),
        second: d.getSeconds(),
        dayOfWeek: d.getDay()
    };
}
exports.$components = $components;
function $componentsarevalid(comp) {
    if (!(0, commons_1.$ok)(comp)) {
        return false;
    }
    ;
    const c = comp;
    return (0, tsdate_1.$dayisvalid)(c.year, c.month, c.day) && (0, tsdate_1.$timeisvalid)(c.hour, c.minute, c.second);
}
exports.$componentsarevalid = $componentsarevalid;
function $componentshavetime(c) {
    return c.hour > 0 || c.minute > 0 || c.second > 0;
}
exports.$componentshavetime = $componentshavetime;
function $parsetime(s) {
    if (!(0, commons_1.$length)(s)) {
        return null;
    }
    const m = s.match(/^\s*(\d{1,6})(\s*[:.]\s*(\d{1,2})(\s*[:.]\s*(\d{1,2}))?)?\s*$/);
    if (!(0, commons_1.$ok)(m)) {
        return null;
    }
    const res = m;
    const c = { hour: types_1.UINT_MIN, minute: types_1.UINT_MIN, second: types_1.UINT_MIN };
    _parseTime(c, res, 0, res.length);
    return (0, tsdate_1.$timeisvalid)(c.hour, c.minute, c.second) ? c : null;
}
exports.$parsetime = $parsetime;
/**
 * These 2 functions parse a date or a date time.
 * Dependending on which date-form you choose to parse the given string, you may ommit
 * the month or the year, the system will autocomplete the components itself ; you
 * also may enter the year with 2 or 4 digits and the system will try to interpret
 * all 2 digits years as years from the previous or the current century. The limit
 * for that is : all date that seems to be more than 20 years in the future will
 * be considered as a previous century year. Note that if you date is not complete
 * you won't be able to enter a time.
 *
 * If you want to create a TSDate from a parsed string, you don't need this
 * function. Simply call :
 *
 *   myDate = TSDate.fromString(myStringToParse [, dateForm]) ; // this does not throw but return can be null
 *
 * ==WARNING== dayOfWeek is not initialized after parsing.
 */
function $parsedatetime(s, form = TSDateForm.Standard) {
    return _parsedt(s, /^\s*(\d{1,8})([/\-.](\d{1,2})([/\-.](\d{1,4})(\s+(\d{1,6})(\s*[:.]\s*(\d{1,2})(\s*[:.]\s*(\d{1,2}))?)?)?)?)?\s*$/, form);
}
exports.$parsedatetime = $parsedatetime;
function $parsedate(s, form = TSDateForm.Standard) {
    return _parsedt(s, /^\s*(\d{1,8})([/\-.](\d{1,2})([/\-.](\d{1,4}))?)?\s*$/, form, { noTime: true });
}
exports.$parsedate = $parsedate;
const NO_TIME_ISO_REGEX = /^([0-9]{2,6})\-([0-9]{1,2})\-([0-9]{1,2})$/;
const TIME_ISO_REGEX = /^([0-9]{2,6})\-([0-9]{1,2})\-([0-9]{1,2})([Tt]([0-9]{1,2})(:([0-9]{1,2})(:([0-9]{1,2})(\.000)?)?)?([zZ]|\+00(:?00)?)?)?$/;
const COMPACT_NO_TIME_ISO_REGEX = /^([0-9]{4})([0-9]{2})([0-9]{2})$/;
const COMPACT_TIME_ISO_REGEX = /^([0-9]{4})([0-9]{2})([0-9]{2})([Tt]([0-9]{2})(([0-9]{2})(([0-9]{2})(\.000)?)?)?([zZ]|\+00(:?00)?)?)?$/;
function $isostring2components(source, opts = {}) {
    const s = (0, commons_1.$ftrim)(source);
    if (!s.length) {
        return null;
    }
    let m = undefined;
    if (opts.noTime) {
        m = s.match(NO_TIME_ISO_REGEX);
        if (!(0, commons_1.$ok)(m)) {
            m = s.match(COMPACT_NO_TIME_ISO_REGEX);
        }
    }
    else {
        m = s.match(TIME_ISO_REGEX);
        if (!(0, commons_1.$ok)(m)) {
            m = s.match(COMPACT_TIME_ISO_REGEX);
        }
    }
    if (!(0, commons_1.$ok)(m)) {
        return null;
    }
    const res = m;
    const l = res.length;
    const c = {
        year: _yearFrom(res[1]),
        month: (0, commons_1.$unsigned)(res[2]),
        day: (0, commons_1.$unsigned)(res[3]),
        hour: l > 5 ? (0, commons_1.$unsigned)(res[5]) : types_1.UINT_MIN,
        minute: l > 7 ? (0, commons_1.$unsigned)(res[7]) : types_1.UINT_MIN,
        second: l > 9 ? (0, commons_1.$unsigned)(res[9]) : types_1.UINT_MIN
    };
    return $componentsarevalid(c) ? c : null;
}
exports.$isostring2components = $isostring2components;
function $components2timestamp(c) {
    return (0, tsdate_1.$timestamp)(c.year, c.month, c.day, c.hour, c.minute, c.second);
}
exports.$components2timestamp = $components2timestamp;
function $components2date(c) {
    return new Date(c.year, c.month - 1, c.day, c.hour, c.minute, c.second, 0);
}
exports.$components2date = $components2date;
function $components2string(c, form = TSDateForm.Standard) {
    switch (form) {
        case TSDateForm.ISO8601:
            return `${(0, commons_1.$fpad4)(c.year)}-${(0, commons_1.$fpad2)(c.month)}-${(0, commons_1.$fpad2)(c.day)}T${(0, commons_1.$fpad2)(c.hour)}:${(0, commons_1.$fpad2)(c.minute)}:${(0, commons_1.$fpad2)(c.second)}`;
        case TSDateForm.ISO8601C:
            return `${(0, commons_1.$fpad4)(c.year)}${(0, commons_1.$fpad2)(c.month)}${(0, commons_1.$fpad2)(c.day)}T${(0, commons_1.$fpad2)(c.hour)}${(0, commons_1.$fpad2)(c.minute)}${(0, commons_1.$fpad2)(c.second)}`;
        case TSDateForm.ISO8601L:
            return `${(0, commons_1.$fpad)(c.year, 6)}-${(0, commons_1.$fpad2)(c.month)}-${(0, commons_1.$fpad2)(c.day)}T${(0, commons_1.$fpad2)(c.hour)}:${(0, commons_1.$fpad2)(c.minute)}:${(0, commons_1.$fpad2)(c.second)}`;
        case TSDateForm.Standard:
            return $componentshavetime(c) ? `${(0, commons_1.$fpad2)(c.day)}/${(0, commons_1.$fpad2)(c.month)}/${(0, commons_1.$fpad4)(c.year)}-${(0, commons_1.$fpad2)(c.hour)}:${(0, commons_1.$fpad2)(c.minute)}:${(0, commons_1.$fpad2)(c.second)}`
                : `${(0, commons_1.$fpad2)(c.day)}/${(0, commons_1.$fpad2)(c.month)}/${(0, commons_1.$fpad4)(c.year)}`;
        case TSDateForm.English:
            return $componentshavetime(c) ? `${(0, commons_1.$fpad2)(c.month)}/${(0, commons_1.$fpad2)(c.day)}/${(0, commons_1.$fpad4)(c.year)}-${(0, commons_1.$fpad2)(c.hour)}:${(0, commons_1.$fpad2)(c.minute)}:${(0, commons_1.$fpad2)(c.second)}`
                : `${(0, commons_1.$fpad2)(c.month)}/${(0, commons_1.$fpad2)(c.day)}/${(0, commons_1.$fpad4)(c.year)}`;
        case TSDateForm.Computer:
            return $componentshavetime(c) ? `${(0, commons_1.$fpad4)(c.year)}/${(0, commons_1.$fpad2)(c.month)}/${(0, commons_1.$fpad4)(c.day)}-${(0, commons_1.$fpad2)(c.hour)}:${(0, commons_1.$fpad2)(c.minute)}:${(0, commons_1.$fpad2)(c.second)}`
                : `${(0, commons_1.$fpad4)(c.year)}/${(0, commons_1.$fpad2)(c.month)}/${(0, commons_1.$fpad4)(c.day)}`;
    }
}
exports.$components2string = $components2string;
;
function $components2StringWithOffset(c, opts = {}) {
    if (!(0, commons_1.$ok)(opts.form)) {
        opts.form = TSDateForm.ISO8601;
    }
    if (!(0, commons_1.$ok)(opts.minutesOffset)) {
        opts.minutesOffset = 0;
    }
    if (!(0, commons_1.$isint)(opts.minutesOffset)) {
        throw '$components2isodateString(): output time zone offset must be a plain unsigned integer';
    }
    if ((0, commons_1.$ok)(opts.milliseconds) && !(0, commons_1.$isunsigned)(opts.milliseconds, 999)) {
        throw '$components2isodateString(): milliseconds offset should be in 0...999 range';
    }
    const m = opts.minutesOffset;
    const a = Math.abs(m);
    if (a % 15 !== 0) {
        throw '$components2isodateString(): output time zone offset must be a multiple of 15';
    }
    let s = $components2string(c, opts.form);
    if ((0, commons_1.$ok)(opts.milliseconds)) {
        s += '.' + (0, commons_1.$fpad3)(opts.milliseconds);
    }
    if (opts.form === TSDateForm.ISO8601C) {
        // in short form, we use Z, +HH and +HHMM offsets
        if (!a) {
            return s + 'Z';
        }
        return a % 60 === 0 ? `${s}${m < 0 ? '-' : '+'}${(0, commons_1.$fpad2)((0, commons_1.$div)(a, 60))}` :
            `${s}${m < 0 ? '-' : '+'}${(0, commons_1.$fpad2)((0, commons_1.$div)(a, 60))}${(0, commons_1.$fpad2)((a % 60))}`;
    }
    if (!a && opts.forceZ) {
        return s + 'Z';
    }
    // other forms commes with +HH:MM offsets, even for hour's round offsets
    return `${s}${m < 0 ? '-' : '+'}${(0, commons_1.$fpad2)((0, commons_1.$div)(a, 60))}:${(0, commons_1.$fpad2)((a % 60))}`;
}
exports.$components2StringWithOffset = $components2StringWithOffset;
/**
 *      date format documentation :
 *
 *      %Y  year padded to 4 digits
 *      %y  year paddeed to 2 digits (centuries are ommited)
 *      %z  non padded year
 *
 *      %m  month padded to 2 digits
 *      %n  not padded month
 *      %b  short month name
 *      %B  month name
 *
 *      %d  day padded to 2 digits
 *      %e  not padded day
 *
 *      %f  day of week (0 = sunday, 1 = monday, ..., 6 = Saturday)
 *      %F  day of week calculated with the current local starting day (0 = monday in France)
 *      %a  short day of week name
 *      %A  day of week name
 *
 *      %q  day of year padded to 3 digits
 *      %r  not padded day of year
 *
 *      %H  24 hour representation padded to 2 digits
 *      %I  12 hour representation padded to 2 digits
 *      %J  not padded 24 hour representation of hours
 *      %K  not padded 12 hour representation of hours
 *      %M  minutes padded to 2 digits
 *      %N  not padded minutes
 *      %P  AM/PM
 *      %S  seconds padded to 2 digits
 *      %T  not padded seconds
 *
 *      %p  standard partial time format for the given language (time without seconds)
 *      %t  standard time format for the given language
 *
 *      %v  week of year padded to 2 digits
 *      %w  not padded week of year
 *
 *      %x  standard short date format for the given language
 *      %X  standard date format for the given language
 *
 *      %%  % caracter
 *      %[  enters a format zone which will be disabled if the passed date has no time
 *          (meaning enabled if hour > 0 || minute > 0 || second > 0)
 *      %]  exit previous format zone without time
 */
function $components2stringformat(comp, format = '', locale) {
    if (!$componentsarevalid(comp)) {
        return null;
    }
    const ts = $components2timestamp(comp);
    const trs = !(0, commons_1.$ok)(locale) || (0, commons_1.$isstring)(locale) || (locale instanceof tscountry_1.TSCountry) ? (0, tsdefaults_1.$locales)(locale) : locale;
    let fmtlen = (0, commons_1.$length)(format);
    let ret = "";
    let escape = false;
    let skipIncludedIfNoTime = false;
    if (!fmtlen) {
        format = trs.dateTimeFormat;
    }
    else {
        switch (format) {
            case TSDateRep.LocalDate:
                format = trs.dateFormat;
                break;
            case TSDateRep.ShortLocalDate:
                format = trs.shortDateFormat;
                break;
            case TSDateRep.LocalDateTime:
                format = trs.dateTimeFormat;
                break;
            case TSDateRep.ShortLocalDateTime:
                format = trs.shortDateTimeFormat;
                break;
            case TSDateRep.LocalTime:
                format = trs.timeFormat;
                break;
            default:
                break;
        }
    }
    fmtlen = format.length;
    for (let i = 0; i < fmtlen; i++) {
        const c = format.charAt(i);
        if (escape) {
            escape = false;
            if (skipIncludedIfNoTime) {
                if (c === ']') {
                    skipIncludedIfNoTime = false;
                }
            }
            else {
                switch (c) {
                    case '%':
                        ret += '%';
                        break;
                    case 'a':
                        ret += trs.shortDays[(0, tsdate_1.$dayOfWeekFromTimestamp)(ts)];
                        break;
                    case 'A':
                        ret += trs.days[(0, tsdate_1.$dayOfWeekFromTimestamp)(ts)];
                        break;
                    case 'b':
                        ret += trs.shortMonths[comp.month - 1];
                        break;
                    case 'B':
                        ret += trs.months[comp.month - 1];
                        break;
                    case 'd':
                        ret += (0, commons_1.$fpad2)(comp.day);
                        break;
                    case 'e':
                        ret += comp.day;
                        break;
                    case 'f':
                        ret += (0, tsdate_1.$dayOfWeekFromTimestamp)(ts);
                        break;
                    case 'F':
                        ret += (0, tsdate_1.$dayOfWeekFromTimestamp)(ts, trs.startingWeekDay);
                        break;
                    case 'H':
                        ret += (0, commons_1.$fpad2)(comp.hour);
                        break;
                    case 'J':
                        ret += (0, commons_1.$fpad2)((comp.hour > 12 ? comp.hour - 12 : comp.hour));
                        break;
                    case 'I':
                        ret += comp.hour;
                        break;
                    case 'K':
                        ret += comp.hour > 12 ? comp.hour - 12 : comp.hour;
                        break;
                    case 'm':
                        ret += (0, commons_1.$fpad2)(comp.month);
                        break;
                    case 'M':
                        ret += (0, commons_1.$fpad2)(comp.minute);
                        break;
                    case 'N':
                        ret += comp.minute;
                        break;
                    case 'n':
                        ret += comp.month;
                        break;
                    case 'p': {
                        const s = $components2stringformat(comp, trs.partialTimeFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'P':
                        ret += trs.ampm[comp.hour <= 12 ? 0 : 1];
                        break;
                    case 'q':
                        ret += (0, commons_1.$fpad3)((0, tsdate_1.$dayOfYear)(ts));
                        break;
                    case 'r':
                        ret += (0, tsdate_1.$dayOfYear)(ts);
                        break;
                    case 'S':
                        ret += (0, commons_1.$fpad2)(comp.second);
                        break;
                    case 'T':
                        ret += comp.second;
                        break;
                    case 't': {
                        const s = $components2stringformat(comp, trs.timeFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'v':
                        ret += (0, commons_1.$fpad2)((0, tsdate_1.$weekOfYear)(ts, trs.startingWeekDay));
                        break;
                    case 'w':
                        ret += (0, tsdate_1.$weekOfYear)(ts, trs.startingWeekDay);
                        break;
                    case 'x': {
                        const s = $components2stringformat(comp, trs.shortDateFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'X': {
                        const s = $components2stringformat(comp, trs.dateFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'y':
                        ret += (0, commons_1.$fpad2)((comp.year % 100));
                        break;
                    case 'Y':
                        ret += (0, commons_1.$fpad4)(comp.year);
                        break;
                    case 'z':
                        ret += comp.year;
                        break;
                    case '[':
                        skipIncludedIfNoTime = comp.hour == 0 && comp.minute == 0 && comp.second == 0;
                        break;
                    case ']':
                        skipIncludedIfNoTime = false;
                        break;
                    default:
                        ret += '%';
                        ret += c;
                        break;
                }
            }
        }
        else if (c === '%') {
            escape = true;
        }
        else {
            escape = false;
            if (!skipIncludedIfNoTime) {
                ret += c;
            }
        }
    }
    if (escape && !skipIncludedIfNoTime) {
        ret += '%';
    }
    return ret;
}
exports.$components2stringformat = $components2stringformat;
function $durationcomponents(duration) {
    if ((0, commons_1.$ok)(duration) && duration < 0) {
        throw '$durationcomponents() : duration must be positive or 0';
    }
    let time = (0, commons_1.$tounsigned)(duration); // we trash subseconds duration digits
    let d, h, m;
    d = (0, commons_1.$div)(time, tsdate_1.TSDay);
    time -= d * tsdate_1.TSDay;
    h = (0, commons_1.$div)(time, tsdate_1.TSHour);
    time -= h * tsdate_1.TSHour;
    m = (0, commons_1.$div)(time, tsdate_1.TSMinute);
    return {
        days: d,
        hours: h,
        minutes: m,
        seconds: (time - m * tsdate_1.TSMinute)
    };
}
exports.$durationcomponents = $durationcomponents;
function $duration(comps) {
    return comps.days * tsdate_1.TSDay + comps.hours * tsdate_1.TSHour + comps.minutes * tsdate_1.TSMinute + comps.seconds;
}
exports.$duration = $duration;
function $duration2String(comps, format) {
    // we reexport in number before constructing the string in order
    // to normalize the number of days, hours, minutes and seconds
    return $durationNumber2StringFormat($duration(comps), format);
}
exports.$duration2String = $duration2String;
/**
 *      duration Format documentation :
 *
 *      %%  % character
 *
 *      %d  non formated days duration
 *      %D  2-digits formated days duration
 *      %E  3-digits formated days duration
 *      %(  introducing conditional part of format valid only if days > 0
 *          only %%, %), %p, %[ %{, %<, %≤, %d, %D and %E can be used in this
 *          conditional format part. All other escape schemas are copied as it
 *          in the final string
 *      %)  terminating conditional part of format valid only if days > 0
 *      %p  else of the conditional part of parenthesis conditional part. In that else
 *          part only text and %%, %p, %) sequences are allowed
 *
 *      %h  non formated hours (< 24h) duration
 *      %H  2-digits formated hours (< 24h) duration
 *      %i  non formated hours (including days) duration
 *      %I  2 digit formated hours (including days) duration
 *      %J  3 digit formated hours (including days) duration
 *
 *      %m  non formated minutes duration
 *      %M  2-digits formated minutes duration
 *
 *      %s  non formated seconds duration
 *      %S  2-digits formated seconds duration
 *      %{  introducing conditional part of format valid only if seconds > 0
 *          only %%, %}, %b, %(, %[, %<, %≤, %s, %S can be used in this conditional
 *          format part. All other escape schemas are copied as it in the final string
 *      %}  terminating conditional part of format valid only if seconds > 0
 *      %b  else of the conditional part of curly bracket conditional part. In that else
 *          part only text and %%, %b, %} sequences are allowed
 *
 *      %[  introducing conditional part of format valid only if we have a duration
 *          where we have seconds, minutes or hours defined. In this conditional part
 *          only %], %%, %!, %(, %{, %<, %≤, %h, %H, %i, %I, %J, %m, %M, %s, %S can
 *          be used. All other escape sequences are copied as it in the final string
 *      %]  ending precedent conditional part
 *      %!  else of the conditional part of bracket conditional part. In that else
 *          part only text and %%, %!, %] sequences are allowed
 *
 *      %≤  introducing conditional part of format valid only if hours > 0. Only %≥, %%, %q, %<,
 *          %(, %[, %{, %h, %H, %i, %I, %J escape sequence can be used here. All other escape
 *          sequences are copied as it in the final string
 *      %≥  ending precedent conditional part
 *      %q  else of the precedent conditional part. In that else part, only text and %%, %q,
 *          and %≥ sequences are allowed
 *
 *      %<  introducing conditional part of format valid only if minutes > 0 or seconds > 0
 *          only %%, %c, %>, %(, %[, %{, %≤, %m, %M, %s, %S can be used in this conditional
 *          format part. All other escape schemas are copied as it in the final string
 *      %>  terminating conditional part of format valid only if minutes > 0 or seconds > 0
 *      %c  else of the conditional part of comparison bracket conditional part. In that else
 *          part only text and %%, %c, %> sequences are allowed
 *
 *      You can use $setdefault('debugDurationStateAutomat', true) ; before calling the present
 *      function in order to debug the format automat if you think you don't obtain what you should
 */
function $durationNumber2StringFormat(duration, format) {
    let State;
    (function (State) {
        State[State["Standard"] = 0] = "Standard";
        State[State["StandardEscape"] = 1] = "StandardEscape";
        State[State["DaysPart"] = 2] = "DaysPart";
        State[State["DaysPartEscape"] = 3] = "DaysPartEscape";
        State[State["HoursPart"] = 4] = "HoursPart";
        State[State["HoursPartEscape"] = 5] = "HoursPartEscape";
        State[State["SecondsPart"] = 6] = "SecondsPart";
        State[State["SecondsPartEscape"] = 7] = "SecondsPartEscape";
        State[State["SubdaysPart"] = 8] = "SubdaysPart";
        State[State["SubdaysPartEscape"] = 9] = "SubdaysPartEscape";
        State[State["SubhoursPart"] = 10] = "SubhoursPart";
        State[State["SubhoursPartEscape"] = 11] = "SubhoursPartEscape";
    })(State || (State = {}));
    ;
    const comp = $durationcomponents(duration);
    if (!(0, commons_1.$length)(format)) {
        format = "%(%d-%)%H:%M%{:%S%}";
    }
    let fmtlen = format.length;
    let state = State.Standard;
    let ret = "";
    let elsePart = false;
    const subhour = comp.minutes > 0 || comp.seconds > 0;
    const subday = comp.hours > 0 || subhour;
    let stack = [];
    const debug = (0, tsdefaults_1.$default)("debugDurationStateAutomat");
    if (debug) {
        console.log(`Format:      "${format}"`);
        console.log(`comp.days:    ${comp.days}`);
        console.log(`comp.hours:   ${comp.hours}`);
        console.log(`comp.minutes: ${comp.minutes}`);
        console.log(`comp.seconds: ${comp.seconds}`);
        console.log(`subhour:      ${subhour}`);
        console.log(`subday:       ${subday}`);
    }
    function _pop() { const p = stack.pop(); return [p.state, p.elsePart]; }
    function _push(newState) { stack.push({ state: state, elsePart: elsePart }); return [newState, false]; }
    function _default(c) { ret += '%'; if (c !== '%') {
        ret += c;
    } }
    function _d() { ret += comp.days; }
    function _D() { ret += (0, commons_1.$fpad2)(comp.days); }
    function _E() { ret += (0, commons_1.$fpad3)(comp.days); }
    function _h() { ret += comp.hours; }
    function _H() { ret += (0, commons_1.$fpad2)(comp.hours); }
    function _i() { ret += (comp.days * 24 + comp.hours); }
    function _I() { ret += (0, commons_1.$fpad2)((comp.days * 24 + comp.hours)); }
    function _J() { ret += (0, commons_1.$fpad3)((comp.days * 24 + comp.hours)); }
    function _m() { ret += comp.minutes; }
    function _M() { ret += (0, commons_1.$fpad2)(comp.minutes); }
    function _s() { ret += comp.seconds; }
    function _S() { ret += (0, commons_1.$fpad2)(comp.seconds); }
    for (let i = 0; i < fmtlen; i++) {
        const c = format.charAt(i);
        if (debug)
            console.log(`stack.count = ${stack.length}, state = ${state}, elsePart = ${elsePart}, char[${i}] = '${c}'`);
        switch (state) {
            case State.Standard:
                elsePart = false;
                if (c === '%') {
                    state = State.StandardEscape;
                }
                else {
                    ret += c;
                }
                break;
            case State.StandardEscape:
                state = State.Standard;
                switch (c) {
                    case '%':
                        ret += '%';
                        break;
                    case '(':
                        [state, elsePart] = _push(State.DaysPart);
                        break;
                    case '{':
                        [state, elsePart] = _push(State.SecondsPart);
                        break;
                    case '[':
                        [state, elsePart] = _push(State.SubdaysPart);
                        break;
                    case '≤':
                        [state, elsePart] = _push(State.HoursPart);
                        break;
                    case '<':
                        [state, elsePart] = _push(State.SubhoursPart);
                        break;
                    case 'D':
                        _D();
                        break;
                    case 'd':
                        _d();
                        break;
                    case 'E':
                        _E();
                        break;
                    case 'H':
                        _H();
                        break;
                    case 'h':
                        _h();
                        break;
                    case 'I':
                        _I();
                        break;
                    case 'i':
                        _i();
                        break;
                    case 'J':
                        _J();
                        break;
                    case 'M':
                        _M();
                        break;
                    case 'm':
                        _m();
                        break;
                    case 'S':
                        _S();
                        break;
                    case 's':
                        _s();
                        break;
                    default:
                        ret += '%', ret += c;
                        break;
                }
                break;
            case State.DaysPart:
                if (c === '%') {
                    state = State.DaysPartEscape;
                }
                else if ((comp.days > 0 && !elsePart) || (!comp.days && elsePart)) {
                    ret += c;
                }
                break;
            case State.DaysPartEscape:
                if (c === ')') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = state = State.DaysPart;
                    if (c === 'p') {
                        elsePart = !elsePart;
                    }
                    else if (comp.days > 0 && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'D':
                                _D();
                                break;
                            case 'd':
                                _d();
                                break;
                            case 'E':
                                _E();
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (elsePart && !comp.days) {
                        _default(c);
                    }
                }
                break;
            case State.HoursPart:
                if (c === '%') {
                    state = State.HoursPartEscape;
                }
                else if ((comp.hours > 0 && !elsePart) || (!comp.hours && elsePart)) {
                    ret += c;
                }
                break;
            case State.HoursPartEscape:
                if (c === '≥') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.HoursPart;
                    if (c === 'q') {
                        elsePart = !elsePart;
                    }
                    else if (comp.hours > 0 && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'H':
                                _H();
                                break;
                            case 'h':
                                _h();
                                break;
                            case 'I':
                                _I();
                                break;
                            case 'i':
                                _i();
                                break;
                            case 'J':
                                _J();
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (elsePart && !comp.hours) {
                        _default(c);
                    }
                }
                break;
            case State.SecondsPart:
                if (c === '%') {
                    state = State.SecondsPartEscape;
                }
                else if ((comp.seconds > 0 && !elsePart) || (!comp.seconds && elsePart)) {
                    ret += c;
                }
                break;
            case State.SecondsPartEscape:
                if (c === '}') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.SecondsPart;
                    if (c === 'b') {
                        elsePart = !elsePart;
                    }
                    else if (comp.seconds > 0 && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'S':
                                _S();
                                break;
                            case 's':
                                _s();
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (elsePart && !comp.seconds) {
                        _default(c);
                    }
                }
                break;
            case State.SubdaysPart:
                if (c === '%') {
                    state = State.SubdaysPartEscape;
                }
                else if ((subday && !elsePart) || (!subday && elsePart)) {
                    ret += c;
                }
                break;
            case State.SubdaysPartEscape:
                if (c === ']') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.SubdaysPart;
                    if (c === '!') {
                        elsePart = !elsePart;
                    }
                    else if (subday && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'H':
                                _H();
                                break;
                            case 'h':
                                _h();
                                break;
                            case 'I':
                                _I();
                                break;
                            case 'i':
                                _i();
                                break;
                            case 'J':
                                _J();
                                break;
                            case 'M':
                                _M();
                                break;
                            case 'm':
                                _m();
                                break;
                            case 'S':
                                _S();
                                break;
                            case 's':
                                _s();
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (!subday && elsePart) {
                        _default(c);
                    }
                }
                break;
            case State.SubhoursPart:
                if (c === '%') {
                    state = State.SubhoursPartEscape;
                }
                else if ((subhour && !elsePart) || (!subhour && elsePart)) {
                    ret += c;
                }
                break;
            case State.SubhoursPartEscape:
                if (c === '>') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.SubhoursPart;
                    if (c === 'c') {
                        elsePart = !elsePart;
                    }
                    else if (subhour && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'M':
                                _M();
                                break;
                            case 'm':
                                _m();
                                break;
                            case 'S':
                                _S();
                                break;
                            case 's':
                                _s();
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (!subhour && elsePart) {
                        _default(c);
                    }
                }
                break;
        }
        if (debug) {
            console.log(`state after:${state}, composed string = "${ret}"`);
        }
    }
    switch (state) {
        case State.Standard:
        case State.DaysPart:
        case State.SecondsPart:
        case State.SubdaysPart:
        case State.SubhoursPart:
        case State.HoursPart:
            break;
        case State.StandardEscape:
            ret += '%';
            break;
        case State.DaysPartEscape:
            if ((comp.days > 0 && !elsePart) || (!comp.days && elsePart)) {
                ret += '%';
            }
            break;
        case State.HoursPartEscape:
            if ((comp.hours > 0 && !elsePart) || (!comp.hours && elsePart)) {
                ret += '%';
            }
            break;
        case State.SecondsPartEscape:
            if ((comp.seconds > 0 && !elsePart) || (!comp.days && elsePart)) {
                ret += '%';
            }
            break;
        case State.SubdaysPartEscape:
            if ((subday && !elsePart) || (!subday && elsePart)) {
                ret += '%';
            }
            break;
        case State.SubhoursPartEscape:
            if ((subhour && !elsePart) || (!subhour && elsePart)) {
                ret += '%';
            }
            break;
    }
    if (debug) {
        console.log(`$durationNumber2StringFormat() returned string = "${ret}"`);
    }
    return ret;
}
exports.$durationNumber2StringFormat = $durationNumber2StringFormat;
//////////////////// private functions
function _yearFrom(ys) {
    const y = (0, commons_1.$unsigned)(ys);
    return ys.length > 2 ? y : _adjustYear(y);
}
/**
 * This function adjust the year entered with only 2 digits
 * follling the following rule :
 *  - each date which seems to be more than 20 years in the future is considered as a previous century date
 *  - each date which seems to be less than 20 years in the future is considered as a current century date
 */
function _adjustYear(y) {
    const currentYear = new Date().getFullYear();
    const century = (0, commons_1.$div)(currentYear, 100);
    return (y > currentYear % 100 + 20 ? y + (century - 1) * 100 : y + century * 100);
}
function _completeWithToday(c) {
    if (c.day) {
        if (c.month) {
            if (!c.year) {
                c.year = (new Date().getFullYear());
            }
        }
        else if (!c.year) {
            const d = new Date();
            c.month = (d.getMonth() + 1);
            c.year = d.getFullYear();
        }
    }
}
function _parsedt(s, regexp, form = TSDateForm.Standard, opts = {}) {
    if (form === TSDateForm.ISO8601 || form === TSDateForm.ISO8601C || form === TSDateForm.ISO8601L) {
        return $isostring2components(s, opts);
    }
    if (!(0, commons_1.$length)(s)) {
        return null;
    }
    const m = s.match(regexp);
    if (!(0, commons_1.$ok)(m)) {
        return null;
    }
    const res = m;
    const l = res.length;
    const c = { year: types_1.UINT_MIN, month: types_1.UINT_MIN, day: types_1.UINT_MIN, hour: types_1.UINT_MIN, minute: types_1.UINT_MIN, second: types_1.UINT_MIN };
    switch (form) {
        case TSDateForm.Standard:
            const ds = (0, commons_1.$unsigned)(res[1]);
            if (l === 2) {
                if (ds >= 1010001) {
                    c.year = (ds % 10000);
                    c.month = (0, commons_1.$div)(ds % 1000000, 10000);
                    c.day = (0, commons_1.$div)(ds, 1000000);
                }
                else if (ds >= 10101) {
                    c.year = _adjustYear(ds % 100);
                    c.month = (0, commons_1.$div)(ds % 10000, 100);
                    c.day = (0, commons_1.$div)(ds, 10000);
                }
                else if (ds >= 101) {
                    c.month = (ds % 100);
                    c.day = (0, commons_1.$div)(ds, 100);
                    _completeWithToday(c);
                }
                else {
                    c.day = ds;
                    _completeWithToday(c);
                }
            }
            else {
                c.day = ds;
                c.month = (0, commons_1.$unsigned)(res[3]);
                if (l >= 6) {
                    c.year = _yearFrom(res[5]);
                    if (l > 6) {
                        _parseTime(c, res, 5, l - 5);
                    }
                }
                else {
                    _completeWithToday(c);
                }
            }
            break;
        case TSDateForm.English:
            const me = (0, commons_1.$unsigned)(res[1]);
            if (l === 2) {
                if (me >= 1010001) {
                    c.year = (me % 10000);
                    c.day = (0, commons_1.$div)(me % 1000000, 10000);
                    c.month = (0, commons_1.$div)(me, 1000000);
                }
                else if (me >= 10101) {
                    c.year = _adjustYear(me % 100);
                    c.day = (0, commons_1.$div)(me % 10000, 100);
                    c.month = (0, commons_1.$div)(me, 10000);
                }
                else if (me >= 101) {
                    c.day = (me % 100);
                    c.month = (0, commons_1.$div)(me, 100);
                }
            }
            else {
                c.month = me;
                c.day = (0, commons_1.$unsigned)(res[3]);
                if (l >= 6) {
                    c.year = _yearFrom(res[5]);
                    if (l > 6) {
                        _parseTime(c, res, 5, l - 5);
                    }
                }
                else {
                    _completeWithToday(c);
                }
            }
            break;
        case TSDateForm.Computer:
            const yc = (0, commons_1.$unsigned)(res[1]);
            if (l === 2) {
                if (res[1].length !== 8) {
                    return null;
                } // in computer form we need all the digits
                c.year = (0, commons_1.$div)(yc, 10000);
                c.month = ((0, commons_1.$div)(yc, 100) % 100);
                c.day = (0, commons_1.$div)(yc, 100);
            }
            else if (l >= 6) {
                c.year = yc;
                c.month = (0, commons_1.$unsigned)(res[3]);
                c.day = (0, commons_1.$unsigned)(res[5]);
                if (l > 6) {
                    _parseTime(c, res, 5, l - 5);
                }
            }
            else {
                return null;
            }
            break;
        default:
            return null;
    }
    return $componentsarevalid(c) ? c : null;
}
function _parseTime(c, m, start, len) {
    let v = (0, commons_1.$unsigned)(m[start + 1]);
    if (len === 2) {
        // we have a single number representing the full time
        // like H or HH or HMM or HHMM or HMMSS or HHMMSS
        switch (m[start + 1].length) {
            case 1:
            case 2:
                c.hour = v;
                break;
            case 3:
            case 4:
                c.hour = (0, commons_1.$div)(v, 100);
                c.minute = (v % 100);
                break;
            case 5:
            case 6:
                c.hour = (0, commons_1.$div)(v, 10000);
                c.minute = ((0, commons_1.$div)(v, 100) % 100);
                c.second = (v % 100);
                break;
        }
    }
    else {
        c.hour = v;
        c.minute = (0, commons_1.$unsigned)(m[start + 2]);
        if (len > 4) {
            c.second = (0, commons_1.$unsigned)(m[start + 4]);
        }
    }
}
//# sourceMappingURL=tsdatecomp.js.map