import { country, currency, language, Languages, Nullable, StringDictionary, StringTranslation } from "./types";
import { TSCountry } from "./tscountry";
export interface Locales {
    names: StringTranslation;
    language: language;
    months: string[];
    shortMonths: string[];
    days: string[];
    shortDays: string[];
    startingWeekDay: number;
    dateTimeFormat: string;
    dateFormat: string;
    shortDateFormat: string;
    shortDateTimeFormat: string;
    timeFormat: string;
    partialTimeFormat: string;
    ampm: string[];
}
export declare type StringTranslations = {
    [key in Languages]?: StringDictionary;
};
export declare class TSDefaults {
    private static __instance;
    private static __subfolders;
    /**
     * LOCALES (mostly date/time locales) are set in 7 common EEC languages
     * - english (en)
     * - french (fr),
     * - spanish (es),
     * - german (de),
     * - italian (it)
     * - portuguese (pt)
     * - dutch (nl)
     *
     * This default management system is primarely meant to be used with EEC
     * countries. So we have the 27 countries of the EEC here + some other
     * to complete the panel of the european nations.
     */
    private static __locales;
    defaultPath: string;
    tmpDirectory: string;
    defaultLanguage: language;
    defaultCurrency: currency;
    private _values;
    private _localizations;
    private _countriesMap;
    private _languagesMap;
    private _currenciesMap;
    private _managedLocalesMap;
    private _managedLanguages;
    private constructor();
    managedLanguages(): language[];
    managedLanguage(s?: Nullable<string>): language | null;
    country(s: Nullable<string>): country | null;
    language(s?: Nullable<TSCountry | string>): language | null;
    currency(s?: Nullable<TSCountry | string>): currency | null;
    addLocalizations(lang: language, loc: StringDictionary): void;
    localizations(locale: Nullable<language | country | TSCountry | string>): StringDictionary;
    locales(locale?: Nullable<language | country | TSCountry | string>): Locales;
    static setSubfolders(folders: string[]): void;
    setDefaultLanguage(l: language): language;
    setDefaultCurrency(c: currency): currency;
    setTmpDirectory(path: string): void;
    setValue(key: string, value: any): void;
    getValue(key: string): any;
    static defaults(): TSDefaults;
}
export declare function $localpath(): string;
export declare function $tmp(): string;
export declare function $locales(locale?: Nullable<language | country | TSCountry>): Locales;
export declare function $country(s: Nullable<string>): country | null;
export declare function $language(s?: Nullable<TSCountry | string>): language | null;
export declare function $currency(s?: Nullable<TSCountry | string>): currency | null;
export declare function $default(key: string): any;
export declare function $setdefault(key: string, value?: any): void;
export declare function $removedefault(key: string): void;
