"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.$removedefault = exports.$setdefault = exports.$default = exports.$currency = exports.$language = exports.$country = exports.$locales = exports.$tmp = exports.$localpath = exports.TSDefaults = void 0;
const commons_1 = require("./commons");
const types_1 = require("./types");
const fs_1 = require("./fs");
const os_1 = __importDefault(require("os"));
const tscountry_1 = require("./tscountry");
const locales_json_1 = __importDefault(require("./locales.json"));
const utils_1 = require("./utils");
class TSDefaults {
    constructor() {
        this.tmpDirectory = (0, utils_1.$inbrowser)() ? '' : os_1.default.tmpdir();
        this.defaultLanguage = types_1.Languages.fr;
        this.defaultCurrency = types_1.Currencies.EUR;
        this._values = {};
        this._localizations = {};
        this.defaultPath = __dirname;
        if (!(0, utils_1.$inbrowser)()) {
            for (let sf in TSDefaults.__subfolders) {
                if ((0, fs_1.$filename)(this.defaultPath) === sf) {
                    this.defaultPath = (0, fs_1.$dir)(this.defaultPath);
                }
            }
        }
        this._countriesMap = new Map();
        Object.keys(types_1.Countries).forEach(e => this._countriesMap.set(e, e));
        this._languagesMap = new Map();
        Object.keys(types_1.Languages).forEach(e => this._languagesMap.set(e, e));
        this._currenciesMap = new Map();
        Object.keys(types_1.Currencies).forEach(e => this._currenciesMap.set(e, e));
        this._managedLanguages = TSDefaults.__locales.map(loc => loc.language);
        let managedLocales = new Map();
        TSDefaults.__locales.forEach(loc => {
            managedLocales.set(loc.language, loc);
            this._managedLanguages.forEach(l => managedLocales.set((0, commons_1.$ascii)(loc.names[l].toLowerCase()), loc));
        });
        this._managedLocalesMap = managedLocales;
        tscountry_1.TSCountry.loadCountries(managedLocales, this._managedLanguages);
    }
    managedLanguages() { return [...this._managedLanguages]; } // send a copy
    managedLanguage(s) {
        if (!(0, commons_1.$ok)(s)) {
            return this.defaultLanguage;
        }
        const v = (0, commons_1.$ascii)((0, commons_1.$ftrim)(s).toLowerCase());
        const locales = this._managedLocalesMap.get(v);
        if ((0, commons_1.$ok)(locales)) {
            return locales.language;
        }
        return null;
    }
    country(s) {
        if ((0, commons_1.$ok)(s) && !(0, commons_1.$isstring)(s)) {
            return null;
        }
        const v = (0, commons_1.$ascii)((0, commons_1.$ftrim)(s).toUpperCase());
        const managedCountry = tscountry_1.TSCountry.country(v);
        if ((0, commons_1.$ok)(managedCountry)) {
            return managedCountry.alpha2Code;
        }
        const ret = this._countriesMap.get(v);
        return (0, commons_1.$ok)(ret) ? ret : null;
    }
    language(s) {
        if (!(0, commons_1.$ok)(s)) {
            return this.defaultLanguage;
        }
        if (s instanceof tscountry_1.TSCountry) {
            return s.language;
        }
        const v = (0, commons_1.$ascii)((0, commons_1.$ftrim)(s).toLowerCase());
        const locales = this._managedLocalesMap.get(v);
        if ((0, commons_1.$ok)(locales)) {
            return locales.language;
        }
        const ret = this._languagesMap.get(v);
        return (0, commons_1.$ok)(ret) ? ret : null;
    }
    currency(s) {
        if (!(0, commons_1.$ok)(s)) {
            return this.defaultCurrency;
        }
        if (s instanceof tscountry_1.TSCountry) {
            return s.currency;
        }
        s = (0, commons_1.$ascii)((0, commons_1.$ftrim)(s).toUpperCase());
        const c = tscountry_1.TSCountry.country(s);
        if ((0, commons_1.$ok)(c)) {
            return c.currency;
        }
        const ret = this._currenciesMap.get(s);
        return (0, commons_1.$ok)(ret) ? ret : null;
    }
    addLocalizations(lang, loc) {
        if (!this._managedLocalesMap.get(lang)) {
            throw "Impossible to add localizations to a non managed language";
        }
        if (!(0, commons_1.$isobject)(loc)) {
            throw "Needed to pass a StringDictionary as localization";
        }
        let actualLocalization = this._localizations[lang];
        if (!(0, commons_1.$ok)(actualLocalization)) {
            this._localizations[lang] = Object.assign({}, loc);
        }
        else {
            this._localizations[lang] = Object.assign(Object.assign({}, actualLocalization), loc);
        }
    }
    localizations(locale) {
        if (locale instanceof tscountry_1.TSCountry) {
            return this._localizations[locale.language] || {};
        }
        locale = (0, commons_1.$ftrim)(locale);
        if (locale.length) {
            const lang = this.language(locale);
            if ((0, commons_1.$ok)(lang)) {
                return this._localizations[lang] || {};
            }
            // here we may have a country
            const c = tscountry_1.TSCountry.country(locale);
            if ((0, commons_1.$ok)(c)) {
                return this._localizations[c.language] || {};
            }
        }
        return {};
    }
    locales(locale) {
        if (locale instanceof tscountry_1.TSCountry) {
            return locale.locales;
        }
        locale = (0, commons_1.$ascii)((0, commons_1.$ftrim)(locale));
        if (locale.length) {
            const locales = this._managedLocalesMap.get(locale.toLowerCase());
            if ((0, commons_1.$ok)(locales)) {
                return locales;
            }
            // here we may have a country
            const c = tscountry_1.TSCountry.country(locale);
            if ((0, commons_1.$ok)(c)) {
                return c.locales;
            }
        }
        return this._managedLocalesMap.get(this.defaultLanguage);
    }
    static setSubfolders(folders) {
        this.__subfolders = folders;
    }
    setDefaultLanguage(l) {
        if ((0, commons_1.$ok)(this._managedLocalesMap.get(l))) {
            this.defaultLanguage = l;
        }
        return this.defaultLanguage;
    }
    setDefaultCurrency(c) {
        if ((0, commons_1.$ok)(this._currenciesMap.get(c))) {
            this.defaultCurrency = c;
        }
        return this.defaultCurrency;
    }
    setTmpDirectory(path) {
        if (!(0, utils_1.$inbrowser)()) {
            if ((0, fs_1.$isdirectory)(path)) {
                this.tmpDirectory = path;
            }
        }
    }
    // these 3 methods permits using software to store global values on unique Defaults instance
    setValue(key, value) {
        if ((0, commons_1.$length)(key)) {
            if ((0, commons_1.$ok)(value)) {
                this._values[key] = value;
            }
            else if ((0, commons_1.$ok)(this._values[key])) {
                delete this._values[key];
            }
        }
    }
    getValue(key) { return this._values[key]; }
    static defaults() {
        if (!this.__instance) {
            this.__instance = new TSDefaults();
        }
        return this.__instance;
    }
}
exports.TSDefaults = TSDefaults;
TSDefaults.__subfolders = ['utils', 'tests', 'dist'];
/**
 * LOCALES (mostly date/time locales) are set in 7 common EEC languages
 * - english (en)
 * - french (fr),
 * - spanish (es),
 * - german (de),
 * - italian (it)
 * - portuguese (pt)
 * - dutch (nl)
 *
 * This default management system is primarely meant to be used with EEC
 * countries. So we have the 27 countries of the EEC here + some other
 * to complete the panel of the european nations.
 */
TSDefaults.__locales = locales_json_1.default;
function $localpath() { return TSDefaults.defaults().defaultPath; }
exports.$localpath = $localpath;
function $tmp() { return TSDefaults.defaults().tmpDirectory; }
exports.$tmp = $tmp;
function $locales(locale) { return TSDefaults.defaults().locales(locale); }
exports.$locales = $locales;
function $country(s) { return TSDefaults.defaults().country(s); }
exports.$country = $country;
function $language(s) { return TSDefaults.defaults().language(s); }
exports.$language = $language;
function $currency(s) { return TSDefaults.defaults().currency(s); }
exports.$currency = $currency;
// to get default language, tou call $language() with no parameters or TSDefaults.defaults().defaultLanguage 
// function to manage your own global defaults.
// warning: all of this defaults are stored in memory
function $default(key) { return TSDefaults.defaults().getValue(key); }
exports.$default = $default;
function $setdefault(key, value = undefined) { return TSDefaults.defaults().setValue(key, value); }
exports.$setdefault = $setdefault;
function $removedefault(key) { return TSDefaults.defaults().setValue(key, undefined); }
exports.$removedefault = $removedefault;
//# sourceMappingURL=tsdefaults.js.map