import { TSClone, TSObject } from "./tsobject";
import { Comparison, Nullable } from "./types";
export interface TSPoint {
    x: number;
    y: number;
}
export interface TSSize {
    w: number;
    h: number;
}
export declare enum TSRectEdge {
    TSMinXEdge = 0,
    TSMinYEdge = 1,
    TSMaxXEdge = 2,
    TSMaxYEdge = 3
}
export declare type TSDocumentFormat = 'min' | 'max' | 'letter' | 'legal' | 'tabloid' | 'a0' | 'a1' | 'a2' | 'a3' | 'a4' | 'a5' | 'a6' | 'letter-landscape' | 'legal-landscape' | 'tabloid-landscape' | 'a0-landscape' | 'a1-landscape' | 'a2-landscape' | 'a3-landscape' | 'a4-landscape' | 'a5-landscape' | 'a6-landscape';
export declare const TSDocumentFormats: {
    [key in TSDocumentFormat]: TSSize;
};
export declare class TSRect implements TSPoint, TSSize, TSObject, TSClone<TSRect> {
    x: number;
    y: number;
    w: number;
    h: number;
    constructor();
    constructor(rect: TSRect);
    constructor(format: TSDocumentFormat);
    constructor(arrayRect: number[]);
    constructor(x: number, y: number, w: number, h: number);
    constructor(origin: TSPoint, w: number, h: number);
    constructor(x: number, y: number, size: TSSize | TSDocumentFormat);
    constructor(origin: TSPoint, size: TSSize | TSDocumentFormat);
    get origin(): TSPoint;
    get size(): TSSize;
    get width(): number;
    get height(): number;
    get minX(): number;
    get minY(): number;
    get maxX(): number;
    get maxY(): number;
    get midX(): number;
    get midY(): number;
    get isEmpty(): boolean;
    contains(p: Nullable<TSPoint | TSRect | number[]>): boolean;
    containsPoint(p: Nullable<TSPoint | number[]>): boolean;
    containsRect(p: Nullable<TSRect | number[]>): boolean;
    containedIn(r: Nullable<TSRect | number[]>): boolean;
    containedInRect(r: Nullable<TSRect | number[]>): boolean;
    intersects(r: Nullable<TSRect | number[]>): boolean;
    intersectsRect(r: Nullable<TSRect | number[]>): boolean;
    intersection(r: Nullable<TSRect | number[]>): TSRect;
    intersectionRect(r: Nullable<TSRect | number[]>): TSRect;
    union(r: Nullable<TSRect | number[]>): TSRect;
    unionRect(r: Nullable<TSRect | number[]>): TSRect;
    offset(dx: number, dy: number): TSRect;
    offsetRect(dx: number, dy: number): TSRect;
    inset(dw: number, dh: number): TSRect;
    insetRect(dw: number, dh: number): TSRect;
    integral(): TSRect;
    integralRect(): TSRect;
    /**
     *
     * @param amount the part of widh or height we keep for the slice
     * @param edge on which edge do we decide to divide our rect
     * @returns a tupple with first the slice we want and second the remainding part of the rect
     */
    divide(amount: number, edge: TSRectEdge): [TSRect, TSRect];
    divideRect(amount: number, edge: TSRectEdge): [TSRect, TSRect];
    clone(): TSRect;
    closedPolygon(updatesYCoordinatesfirst?: boolean): TSPoint[];
    isEqual(other: any): boolean;
    compare(other: any): Comparison;
    toJSON(): object;
    toString(): string;
    toArray(): number[];
}
export declare function TSmm2Pixels(mm: number): number;
export declare function TScm2Pixels(cm: number): number;
export declare function TSPixels2cm(pixels: number): number;
export declare function TSPixels2mm(pixels: number): number;
export declare function TSInches2Pixels(inches: number): number;
export declare function TSPixels2Inches(pixels: number): number;
export declare function TSEqualPoints(A: TSPoint, B: TSPoint): boolean;
export declare function TSEqualSizes(A: TSSize, B: TSSize): boolean;
export declare function TSArea(A: TSSize): number;
export declare function TSValidSize(A: Nullable<TSSize>): boolean;
export declare function TSCompareSizes(A: TSSize, B: TSSize): Comparison;
export interface TSAssertFormatOptions {
    defaultSize?: TSSize;
    minimalSize?: TSSize;
    maximalSize?: TSSize;
    invalidSizeRaise?: boolean;
    undersizeRaise?: boolean;
    oversizeRaise?: boolean;
}
export declare function TSAssertFormat(format: Nullable<TSDocumentFormat | TSSize>, opts?: TSAssertFormatOptions): TSSize;
export declare const TSCM: number;
export declare const TSMM: number;
export declare const TSIN: number;
