"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSIN = exports.TSMM = exports.TSCM = exports.TSAssertFormat = exports.TSCompareSizes = exports.TSValidSize = exports.TSArea = exports.TSEqualSizes = exports.TSEqualPoints = exports.TSPixels2Inches = exports.TSInches2Pixels = exports.TSPixels2mm = exports.TSPixels2cm = exports.TScm2Pixels = exports.TSmm2Pixels = exports.TSRect = exports.TSDocumentFormats = exports.TSRectEdge = void 0;
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const types_1 = require("./types");
var TSRectEdge;
(function (TSRectEdge) {
    TSRectEdge[TSRectEdge["TSMinXEdge"] = 0] = "TSMinXEdge";
    TSRectEdge[TSRectEdge["TSMinYEdge"] = 1] = "TSMinYEdge";
    TSRectEdge[TSRectEdge["TSMaxXEdge"] = 2] = "TSMaxXEdge";
    TSRectEdge[TSRectEdge["TSMaxYEdge"] = 3] = "TSMaxYEdge";
})(TSRectEdge = exports.TSRectEdge || (exports.TSRectEdge = {}));
;
exports.TSDocumentFormats = {
    max: { w: TSmm2Pixels(1200), h: TSmm2Pixels(1200) },
    min: { w: TSmm2Pixels(100), h: TSmm2Pixels(100) },
    letter: { w: TSInches2Pixels(8.5), h: TSInches2Pixels(11) },
    legal: { w: TSInches2Pixels(8.5), h: TSInches2Pixels(14) },
    tabloid: { w: TSInches2Pixels(11), h: TSInches2Pixels(17) },
    a0: { w: TSmm2Pixels(841), h: TSmm2Pixels(1189) },
    a1: { w: TSmm2Pixels(594), h: TSmm2Pixels(841) },
    a2: { w: TSmm2Pixels(420), h: TSmm2Pixels(594) },
    a3: { w: TSmm2Pixels(297), h: TSmm2Pixels(420) },
    a4: { w: TSmm2Pixels(210), h: TSmm2Pixels(297) },
    a5: { w: TSmm2Pixels(148), h: TSmm2Pixels(210) },
    a6: { w: TSmm2Pixels(105), h: TSmm2Pixels(148) },
    'letter-landscape': { w: TSInches2Pixels(11), h: TSInches2Pixels(8.5) },
    'legal-landscape': { w: TSInches2Pixels(14), h: TSInches2Pixels(8.5) },
    'tabloid-landscape': { w: TSInches2Pixels(17), h: TSInches2Pixels(11) },
    'a0-landscape': { w: TSmm2Pixels(1189), h: TSmm2Pixels(841) },
    'a1-landscape': { w: TSmm2Pixels(841), h: TSmm2Pixels(594) },
    'a2-landscape': { w: TSmm2Pixels(594), h: TSmm2Pixels(420) },
    'a3-landscape': { w: TSmm2Pixels(420), h: TSmm2Pixels(297) },
    'a4-landscape': { w: TSmm2Pixels(297), h: TSmm2Pixels(210) },
    'a5-landscape': { w: TSmm2Pixels(210), h: TSmm2Pixels(148) },
    'a6-landscape': { w: TSmm2Pixels(148), h: TSmm2Pixels(105) }
};
class TSRect {
    constructor() {
        const n = arguments.length;
        switch (n) {
            case 0:
                this.x = this.y = this.w = this.h = 0;
                break;
            case 1: {
                if ((0, commons_1.$isstring)(arguments[0])) {
                    const size = exports.TSDocumentFormats[arguments[0]];
                    if ((0, commons_1.$ok)(size)) {
                        this.x = this.y = 0;
                        this.w = size.w;
                        this.h = size.h;
                    }
                    else {
                        throw 'Bad TSRect() constructor: bad document format';
                    }
                }
                else if ((0, commons_1.$isarray)(arguments[0])) {
                    const a = arguments[0];
                    if (a.length === 4 && (0, commons_1.$isnumber)(a[0]) && (0, commons_1.$isnumber)(a[1]) && (0, commons_1.$isnumber)(a[2]) && (0, commons_1.$isnumber)(a[3]) && a[2] >= 0 && a[3] >= 0) {
                        this.x = a[0];
                        this.y = a[1];
                        this.w = a[2];
                        this.h = a[3];
                    }
                    else {
                        throw 'Bad TSRect() constructor: bad rect definition array';
                    }
                }
                else if ((arguments[0] instanceof TSRect)) {
                    const r = arguments[0];
                    const x = r.minX;
                    const y = r.minY;
                    const w = r.width;
                    const h = r.height;
                    if ((0, commons_1.$isnumber)(x) && (0, commons_1.$isnumber)(y) && (0, commons_1.$isnumber)(w) && (0, commons_1.$isnumber)(h) && w >= 0 && h >= 0) {
                        this.x = x;
                        this.y = y;
                        this.w = w;
                        this.h = h;
                    }
                    else {
                        throw 'Bad TSRect() constructor: bad TSRect parameter';
                    }
                }
                else {
                    throw 'Bad TSRect() constructor: should have a TSRect or a document format parameter';
                }
                break;
            }
            case 2:
                if ((0, commons_1.$isobject)(arguments[0]) && ('x' in arguments[0]) && ('y' in arguments[0])) {
                    const size = (0, commons_1.$isstring)(arguments[1]) ? exports.TSDocumentFormats[arguments[1]] : arguments[1];
                    if ((0, commons_1.$isobject)(size) && ('w' in size) && ('h' in size) && (0, commons_1.$isnumber)(size.w) && (0, commons_1.$isnumber)(size.h) && size.w >= 0 && size.h >= 0) {
                        this.x = arguments[0].x;
                        this.y = arguments[0].y;
                        this.w = size.w;
                        this.h = size.h;
                    }
                    else {
                        throw 'Bad TSRect() constructor: bad TSSize parameter';
                    }
                }
                else {
                    throw 'Bad TSRect() constructor: bad TSPoint as origin parameter';
                }
                break;
            case 3:
                if ((0, commons_1.$isnumber)(arguments[0]) && (0, commons_1.$isnumber)(arguments[1]) &&
                    (((0, commons_1.$isobject)(arguments[2]) && ('w' in arguments[2]) && ('h' in arguments[2])) || (0, commons_1.$isstring)(arguments[2]))) {
                    const size = (0, commons_1.$isstring)(arguments[2]) ? exports.TSDocumentFormats[arguments[2]] : arguments[2];
                    if ((0, commons_1.$ok)(size) && (0, commons_1.$isnumber)(size.w) && (0, commons_1.$isnumber)(size.h) && size.w >= 0 && size.h >= 0) {
                        this.x = arguments[0];
                        this.y = arguments[1];
                        this.w = size.w;
                        this.h = size.h;
                    }
                    else {
                        throw 'Bad TSRect() constructor: bad TSSize parameter';
                    }
                }
                else if ((0, commons_1.$isobject)(arguments[0]) && ('x' in arguments[0]) && ('y' in arguments[0]) &&
                    (0, commons_1.$isnumber)(arguments[1]) && (0, commons_1.$isnumber)(arguments[2]) && arguments[1] >= 0 && arguments[2] >= 0) {
                    const origin = arguments[0];
                    if ((0, commons_1.$isnumber)(origin.x) && (0, commons_1.$isnumber)(origin.y)) {
                        this.x = origin.x;
                        this.y = origin.y;
                        this.w = arguments[1];
                        this.h = arguments[2];
                    }
                    else {
                        throw 'Bad TSRect() constructor: bad TSPoint origin parameter';
                    }
                }
                else {
                    throw 'Bad TSRect() constructor: bad TSPoint, TSSize or number parameter';
                }
                break;
            case 4:
                if (!(0, commons_1.$isnumber)(arguments[0]) || !(0, commons_1.$isnumber)(arguments[1]) ||
                    !(0, commons_1.$isnumber)(arguments[2]) || !(0, commons_1.$isnumber)(arguments[3]) ||
                    arguments[2] < 0 || arguments[3] < 0) {
                    throw 'Bad TSRect() constructor: bad number parameter';
                }
                this.x = arguments[0];
                this.y = arguments[1];
                this.w = arguments[2];
                this.h = arguments[3];
                break;
            default:
                throw 'Bad TSRect() constructor : too much arguments';
        }
    }
    get origin() { return { x: this.minX, y: this.minY }; }
    get size() { return { w: this.width, h: this.height }; }
    get width() { return this.maxX - this.minX; }
    get height() { return this.maxY - this.minY; }
    get minX() { return this.x; }
    get minY() { return this.y; }
    get maxX() { return this.x + this.w; }
    get maxY() { return this.y + this.h; }
    get midX() { return this.x + this.w / 2; }
    get midY() { return this.y + this.h / 2; }
    get isEmpty() { return this.w > 0 && this.h > 0 ? false : true; }
    contains(p) {
        if (p instanceof TSRect) {
            const r = p;
            return !r.isEmpty && this.minX <= r.minX && this.minY <= r.minY && this.maxX >= r.maxX && this.maxY >= r.maxY;
        }
        if ((0, commons_1.$isarray)(p)) {
            const a = p;
            if ((a.length === 2 || a.length === 4) && (0, commons_1.$isnumber)(a[0]) && (0, commons_1.$isnumber)(a[1]) && (a.length === 2 || ((0, commons_1.$isnumber)(a[2]) && (0, commons_1.$isnumber)(a[3]) && a[2] >= 0 && a[3] >= 0))) {
                return a.length === 2 ? this.contains({ x: a[0], y: a[1] }) : this.contains(new TSRect(a[0], a[1], a[2], a[3]));
            }
            return false;
        }
        return (0, commons_1.$isobject)(p) && ('x' in p) && ('y' in p) ? p.x >= this.minX && p.x <= this.maxX && p.y >= this.minY && p.y <= this.maxY : false;
    }
    containsPoint(p) {
        if ((0, commons_1.$isarray)(p) && p.length !== 2) {
            return false;
        }
        return this.contains(p);
    }
    containsRect(p) {
        if ((0, commons_1.$isarray)(p) && p.length !== 4) {
            return false;
        }
        return this.contains(p);
    }
    containedIn(r) {
        if (!(0, commons_1.$ok)(r)) {
            return false;
        }
        try {
            r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        }
        catch (_a) {
            return false;
        }
        return r.contains(this);
    }
    containedInRect(r) { return this.containedIn(r); }
    intersects(r) {
        if (!(0, commons_1.$ok)(r)) {
            return false;
        }
        try {
            r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        }
        catch (_a) {
            return false;
        }
        return this.maxX <= r.minX || r.maxX <= this.minX || this.maxY <= r.minY || r.maxY <= this.minY || this.isEmpty || r.isEmpty ? false : true;
    }
    intersectsRect(r) { return this.intersects(r); }
    intersection(r) {
        let rect = new TSRect();
        if (!(0, commons_1.$ok)(r)) {
            return rect;
        }
        try {
            r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        }
        catch (_a) {
            return rect;
        }
        if (this.maxX <= r.minX || r.maxX <= this.minX || this.maxY <= r.minY || r.maxY <= this.minY) {
            return rect;
        }
        rect.x = Math.max(this.minX, r.minX);
        rect.y = Math.max(this.minY, r.minY);
        rect.w = Math.min(this.maxX, r.maxX) - rect.x;
        rect.h = Math.min(this.maxY, r.maxY) - rect.y;
        return rect;
    }
    intersectionRect(r) { return this.intersection(r); }
    union(r) {
        if (!(0, commons_1.$ok)(r)) {
            return this.clone();
        }
        r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        if (this.isEmpty) {
            return r.clone();
        } // r may be empty here. We clone it anyway.
        if (r.isEmpty) {
            return this.clone();
        }
        let rect = new TSRect();
        rect.x = Math.min(this.minX, r.minX);
        rect.y = Math.min(this.minY, r.minY);
        rect.w = Math.max(this.maxX, r.maxX) - rect.x;
        rect.h = Math.max(this.maxY, r.maxY) - rect.y;
        return rect;
    }
    unionRect(r) { return this.union(r); }
    offset(dx, dy) {
        if (!(0, commons_1.$isnumber)(dx) || !(0, commons_1.$isnumber)(dy)) {
            throw 'Invalid offser';
        }
        return new TSRect(this.minX + dx, this.minY + dy, this.width, this.height);
    }
    offsetRect(dx, dy) { return this.offset(dx, dy); }
    inset(dw, dh) {
        if (!(0, commons_1.$isnumber)(dw) || !(0, commons_1.$isnumber)(dh)) {
            throw 'Invalid inset';
        }
        return new TSRect(this.minX - dw, this.minY - dh, this.width + dw * 2, this.height + dh * 2);
    }
    insetRect(dw, dh) { return this.inset(dw, dh); }
    integral() {
        let rect = new TSRect();
        rect.x = Math.floor(this.minX);
        rect.y = Math.floor(this.minY);
        if (this.isEmpty) {
            return rect;
        }
        rect.w = Math.ceil(this.maxX) - rect.x;
        rect.h = Math.ceil(this.maxY) - rect.y;
        return rect;
    }
    integralRect() { return this.integral(); }
    /**
     *
     * @param amount the part of widh or height we keep for the slice
     * @param edge on which edge do we decide to divide our rect
     * @returns a tupple with first the slice we want and second the remainding part of the rect
     */
    divide(amount, edge) {
        const x = this.minX;
        const y = this.minY;
        const w = this.width;
        const h = this.height;
        if (amount === Number.POSITIVE_INFINITY) {
            amount = Math.max(w, h);
        }
        if (!(0, commons_1.$isnumber)(amount) || amount < 0) {
            amount = 0;
        }
        switch (edge) {
            case TSRectEdge.TSMinXEdge:
                return amount > w ?
                    [new TSRect(x, y, w, h), new TSRect(this.maxX, y, 0, h)] :
                    [new TSRect(x, y, amount, h), new TSRect(x + amount, y, w - amount, h)];
            case TSRectEdge.TSMinYEdge:
                return amount > h ?
                    [new TSRect(x, y, w, h), new TSRect(x, this.maxY, w, 0)] :
                    [new TSRect(x, y, w, amount), new TSRect(x, y + amount, w, h - amount)];
            case TSRectEdge.TSMaxXEdge:
                return amount > w ?
                    [new TSRect(x, y, w, h), new TSRect(x, y, 0, h)] :
                    [new TSRect(this.maxX - amount, y, amount, h), new TSRect(x, y, w - amount, h)];
            case TSRectEdge.TSMaxYEdge:
                return amount > h ?
                    [new TSRect(x, y, w, h), new TSRect(x, y, w, 0)] :
                    [new TSRect(x, this.maxY - amount, w, amount), new TSRect(x, y, w, h - amount)];
            default:
                // we should never be here, but ...
                throw 'TSRect.divide() : bad edge parameter';
        }
    }
    divideRect(amount, edge) { return this.divide(amount, edge); }
    clone() {
        return new TSRect(this.minX, this.minY, this.width, this.height);
    }
    // this method returns 5 points
    closedPolygon(updatesYCoordinatesfirst = false) {
        const x1 = this.minX, y1 = this.minY, x2 = this.maxX, y2 = this.maxY;
        return updatesYCoordinatesfirst ?
            [{ x: x1, y: y1 }, { x: x1, y: y2 }, { x: x2, y: y2 }, { x: x2, y: y1 }, { x: x1, y: y1 }] :
            [{ x: x1, y: y1 }, { x: x2, y: y1 }, { x: x2, y: y2 }, { x: x1, y: y2 }, { x: x1, y: y1 }];
    }
    // ============ TSObject conformance =============== 
    isEqual(other) { return this === other || (other instanceof TSRect && other.minX === this.minX && other.maxX === this.maxX && other.minY === this.minY && other.maxY === this.maxY); }
    compare(other) {
        if (other === this) {
            return types_1.Same;
        }
        if (!(other instanceof TSRect)) {
            return undefined;
        }
        const area = this.width * this.height;
        const otherArea = other.width * other.height;
        if (area === otherArea) {
            const aX = this.minX;
            const aY = this.minY;
            const bX = other.minX;
            const bY = other.minY;
            if (aX === bX) {
                return aY < bY ? types_1.Ascending : (aY > bY ? types_1.Descending : types_1.Same);
            }
            if (aY === other.y) {
                return aX < bX ? types_1.Ascending : types_1.Descending;
            }
            if (aX < bX && aY < bY) {
                return types_1.Ascending;
            }
            if (aX > bX && aY > bY) {
                return types_1.Descending;
            }
            return undefined; // same area but origins are not comparable
        }
        return area < otherArea ? types_1.Ascending : types_1.Descending;
    }
    toJSON() {
        return { x: this.minX, y: this.minY, w: this.width, h: this.height };
    }
    toString() {
        return `{x = ${this.minX}, y = ${this.minY}), w:${this.width}, h:${this.height}}`;
    }
    // warning : dont use generated array to create new Rect because we did send the oposite points here
    // QUESTION: should we return [x,y,w,h] here ? 
    toArray() { return [this.minX, this.minY, this.maxX, this.maxY]; }
}
exports.TSRect = TSRect;
function TSmm2Pixels(mm) { return mm * 45 / 16; }
exports.TSmm2Pixels = TSmm2Pixels;
function TScm2Pixels(cm) { return cm * 225 / 8; }
exports.TScm2Pixels = TScm2Pixels;
function TSPixels2cm(pixels) { return pixels * 8 / 225; }
exports.TSPixels2cm = TSPixels2cm;
function TSPixels2mm(pixels) { return pixels * 16 / 45; }
exports.TSPixels2mm = TSPixels2mm;
function TSInches2Pixels(inches) { return inches * 72; }
exports.TSInches2Pixels = TSInches2Pixels;
function TSPixels2Inches(pixels) { return pixels / 72; }
exports.TSPixels2Inches = TSPixels2Inches;
function TSEqualPoints(A, B) { return A.x === B.x && A.y === B.y; }
exports.TSEqualPoints = TSEqualPoints;
function TSEqualSizes(A, B) { return A.w === B.w && A.h === B.h; }
exports.TSEqualSizes = TSEqualSizes;
function TSArea(A) { return A.w * A.h; }
exports.TSArea = TSArea;
function TSValidSize(A) { return (0, commons_1.$ok)(A) && A.w >= 0 && A.h >= 0; }
exports.TSValidSize = TSValidSize;
function TSCompareSizes(A, B) { return (0, compare_1.$numcompare)(TSArea(A), TSArea(B)); }
exports.TSCompareSizes = TSCompareSizes;
;
function TSAssertFormat(format, opts = {}) {
    const size = (0, commons_1.$isstring)(format) ? exports.TSDocumentFormats[format] : format;
    const defaultSize = TSValidSize(opts.defaultSize) ? opts.defaultSize : exports.TSDocumentFormats.a4;
    if (!TSValidSize(size)) {
        if (opts.invalidSizeRaise) {
            throw 'TSAssertFormat(): invalid format.';
        }
        return defaultSize;
    }
    const min = TSValidSize(opts.minimalSize) ? opts.minimalSize : exports.TSDocumentFormats.min;
    if (size.w < min.w || size.h < min.h) {
        if (opts.oversizeRaise) {
            throw 'TSAssertFormat(): format too small.';
        }
        return min;
    }
    const max = TSValidSize(opts.maximalSize) ? opts.maximalSize : exports.TSDocumentFormats.max;
    if (size.w > max.w || size.h > max.h) {
        if (opts.oversizeRaise) {
            throw 'TSAssertFormat(): format too large.';
        }
        return max;
    }
    return size;
}
exports.TSAssertFormat = TSAssertFormat;
exports.TSCM = TScm2Pixels(1);
exports.TSMM = TSmm2Pixels(1);
exports.TSIN = TSInches2Pixels(1);
//# sourceMappingURL=tsgeometry.js.map