import { TSClone, TSCollection, TSObject } from "./tsobject";
import { Comparison, Nullable } from "./types";
export declare class TSListNode<T> implements TSObject {
    data: T;
    next: TSListNode<T> | null;
    prev: TSListNode<T> | null;
    constructor(data: T);
    isEqual(other: any): boolean;
    compare(other: any): Comparison;
    toJSON(): any;
    toString(): string;
    toArray(): any[];
}
export interface TSListToStringOptions<T> {
    prefix?: string;
    separator?: string;
    suffix?: string;
    printer?: (data: T) => Nullable<string>;
}
export declare class TSList<T> implements TSObject, TSCollection<T>, TSClone<TSList<T>>, Iterable<T> {
    private _f;
    private _l;
    private _n;
    get length(): number;
    get count(): number;
    get first(): TSListNode<T> | null;
    get last(): TSListNode<T> | null;
    insert(data: T, before?: TSListNode<T>): TSListNode<T>;
    add(data: T): TSListNode<T>;
    removeNode(node: TSListNode<T>): void;
    clear(): void;
    forEach(callback: (data: T) => void): void;
    [Symbol.iterator](): {
        next: () => {
            done: boolean;
            value: any;
        };
    };
    search(callback: (data: T) => boolean): TSListNode<T> | null;
    clone(): TSList<T>;
    isEqual(other: any): boolean;
    compare(other: any): Comparison;
    toString(opts?: TSListToStringOptions<T>): string;
    toJSON(): any[];
    toArray(map?: (data: T) => Nullable<any>): T[];
    getItems(): T[];
}
export interface TSListConstructor<T> {
    new (data: T): TSList<T>;
}
