import { TSRange } from "./tsrange";
import { AnyDictionary, Nullable } from "./types";
import { TSCollection } from "./tsobject";
export declare type QualifierOperator = 'AND' | 'OR' | 'NOT' | 'EQ' | 'NEQ' | 'LT' | 'LTE' | 'GT' | 'GTE' | 'LIKE' | 'IN' | 'NIN' | 'OK' | 'KO';
export declare type QualifierOperand = any;
export declare type QualifierCondition<T> = AnyDictionary | TSQualifier<T>;
export declare const LAZY_INTERSECTION = false;
export declare type TSQualifierCollection<T> = ArrayLike<T> | Set<T> | TSCollection<T>;
export declare type RecursiveKeyPath<T, K extends keyof T> = K extends string ? T[K] extends Record<string, any> ? T[K] extends TSQualifierCollection<infer U> ? K | `${K}.${RecursiveKeyPath<Required<U>, keyof Required<U>>}` : K | `${K}.${RecursiveKeyPath<Required<T[K]>, keyof Required<T[K]>>}` : K : never;
export declare type KeyPath<T> = RecursiveKeyPath<Required<T>, keyof Required<T>> | keyof Required<T>;
export declare class TSQualifier<T> {
    readonly operator: QualifierOperator;
    private _operands;
    protected constructor(operator: QualifierOperator, operands: QualifierOperand[]);
    static OR<T>(conds?: QualifierCondition<T>[]): TSQualifier<T>;
    static AND<T>(conds?: QualifierCondition<T>[]): TSQualifier<T>;
    static NOT<T>(cond: QualifierCondition<T>): TSQualifier<T>;
    static OK<T>(key: KeyPath<T>): TSQualifier<T>;
    static KO<T>(key: KeyPath<T>): TSQualifier<T>;
    static EQ<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static NEQ<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static LT<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static LTE<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static GT<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static GTE<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static LIKE<T>(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static IN<T>(key: KeyPath<T>, values: QualifierOperand[]): TSQualifier<T>;
    static NIN<T>(key: KeyPath<T>, values: QualifierOperand[]): TSQualifier<T>;
    static INCLUDES<T>(key1: KeyPath<T>, key2: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    static INCLUDED<T>(key: KeyPath<T>, value1: QualifierOperand, value2: QualifierOperand, canUnspecify?: boolean, strict?: boolean): TSQualifier<T>;
    static INTERSECTS<T>(key1: KeyPath<T>, key2: KeyPath<T>, value1: QualifierOperand, value2: QualifierOperand, canUnspecify?: boolean, strict?: boolean): TSQualifier<T>;
    static INRANGE<T>(key: KeyPath<T>, range: TSRange | number[]): TSQualifier<T>;
    static intersectionQualifiers<T>(A: QualifierOperand, B: QualifierOperand, C: KeyPath<T>, D: KeyPath<T>, strict: boolean, canUnspecify: boolean): Array<TSQualifier<T>>;
    get isComposite(): boolean;
    get isKeyValue(): boolean;
    key(): string | undefined;
    keyArray(): string[];
    value(): any;
    conditions(): QualifierCondition<T>[];
    condition(cond: QualifierCondition<T>): TSQualifier<T>;
    and(conds?: QualifierCondition<T>[]): TSQualifier<T>;
    or(conds?: QualifierCondition<T>[]): TSQualifier<T>;
    not(condition: QualifierCondition<T>): TSQualifier<T>;
    mayIs(cond: boolean, key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    if(cond: boolean): TSQualifier<T> | undefined;
    is(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    isNot(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    gt(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    lt(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    gte(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    lte(key: KeyPath<T>, value: QualifierOperand): TSQualifier<T>;
    in(key: KeyPath<T>, value: QualifierOperand[]): TSQualifier<T>;
    nin(key: KeyPath<T>, value: QualifierOperand[]): TSQualifier<T>;
    inRange(key: KeyPath<T>, range: TSRange | number[]): TSQualifier<T>;
    ok(key: KeyPath<T>): TSQualifier<T>;
    ko(key: KeyPath<T>): TSQualifier<T>;
    includes(key1: KeyPath<T>, key2: KeyPath<T>, value: any): TSQualifier<T>;
    included(key: KeyPath<T>, value1: any, value2: any, canUnspecify?: boolean, strict?: boolean): TSQualifier<T>;
    intersects(key1: KeyPath<T>, key2: KeyPath<T>, value1: any, value2: any, canUnspecify?: boolean, strict?: boolean): TSQualifier<T>;
    inverse(): TSQualifier<T>;
    validateValue(v: T, validateValueForCondition?: (v: T, cond: AnyDictionary) => boolean): boolean;
    filterValues(values: Nullable<Iterable<T>>): Array<T>;
    private _add;
    private _assertAndOr;
}
export declare function $valuesForKeyPath<T>(v: any, path: KeyPath<T>): any;
declare global {
    export interface String {
        sqlLike: (this: string, like: string) => boolean;
    }
    export interface Array<T> {
        filterWithQualifier: <T>(qual: TSQualifier<T>) => Array<T>;
    }
}
export declare function $sqllike(a: any, b: string): any;
