/// <reference types="node" />
import { IncomingMessage, ServerResponse } from "http";
import { Verb } from "./tsrequest";
import { StringDictionary, TSDictionary, uint16 } from "./types";
import { TSStaticWebSiteOptions } from "./tsservercomp";
import { TSDate } from "./tsdate";
import { TSColor } from "./tscolor";
/**
 * This is a minimal singleton HTTP server class provided for testing
 *
 * You start a new async API server on default port 3000 by a single line :
 *   TSServer.start({ ... my endpoints definitions dictionary ... }) ;
 *
 */
export interface TSServerRequest {
    method: Verb;
    url: URL;
    parameters: TSParameterDictionary;
    query: TSQueryDictionary;
    message: IncomingMessage;
}
export declare type TSServerResponse = ServerResponse;
export declare type TSEndPointManager = (req: TSServerRequest, resp: TSServerResponse) => Promise<void>;
export interface TSEndPoint {
    manager: TSEndPointManager;
    query?: TSQueryDefinition;
}
export declare type TSEndPoints = {
    [key in Verb]?: TSEndPoint | TSEndPointManager;
};
export declare type TSEndPointsDictionary = TSDictionary<TSEndPoints | TSEndPoint | TSEndPointManager>;
export declare type TSEndPointParameter = string | number | boolean | TSDate | TSColor;
export declare type TSParameterDictionary = TSDictionary<TSEndPointParameter>;
export declare type TSQueryValue = TSEndPointParameter;
export declare type TSQueryDictionary = TSDictionary<TSQueryValue>;
export declare const TSParametricTokenType: {
    readonly string: "string";
    readonly number: "number";
    readonly int: "int";
    readonly unsigned: "unsigned";
    readonly boolean: "boolean";
    readonly date: "date";
    readonly color: "color";
    readonly uuid: "uuid";
    readonly email: "email";
};
export declare type TSParametricTokenType = typeof TSParametricTokenType[keyof typeof TSParametricTokenType];
export interface TSParametricToken {
    name: string;
    type: TSParametricTokenType;
}
export interface TSQueryItem {
    type: TSParametricTokenType;
    mandatory?: boolean;
}
export declare type TSQueryDefinition = TSDictionary<TSParametricTokenType | TSQueryItem>;
export interface TSServerOptions extends TSStaticWebSiteOptions {
    host?: string;
    port?: uint16;
    webSites?: StringDictionary;
    logInfos?: boolean;
}
export declare class TSServer {
    readonly host: string;
    readonly port: number;
    private static __server;
    private _httpServer;
    private _endPoints;
    private _sites;
    private _logger;
    private _logInfos;
    static start(endPoints: TSEndPointsDictionary, opts: TSServerOptions): Promise<void>;
    static clearCaches(): Promise<void>;
    static stop(): Promise<Error | undefined>;
    private constructor();
    private _start;
    private _clearCaches;
    private _stop;
}
export declare enum TSServerLogType {
    Log = "Info",
    Warning = "Warning",
    Error = "Error"
}
export declare type TSServerLogger = (server: TSServer, req: IncomingMessage | undefined, type: TSServerLogType, messages: string) => Promise<void>;
