/// <reference types="node" />
/// <reference types="node" />
import { ServerResponse } from "http";
import { Nullable, StringDictionary, TSDictionary, uint32 } from "./types";
import { Verb } from "./tsrequest";
import { TSEndPoint, TSEndPoints, TSParameterDictionary, TSServerLogger, TSParametricToken, TSEndPointManager, TSQueryItem, TSServerRequest } from "./tsserver";
export interface TSStaticWebSiteOptions {
    logger?: TSServerLogger;
    managedTypes?: StringDictionary;
    maxCacheSize?: uint32;
    maxCachedFileSize?: uint32;
    maxCachedFiles?: uint32;
    maxBlacklistedFiles?: uint32;
}
export declare class TSStaticWebsite {
    readonly uri: string;
    readonly folder: string;
    private _cache;
    private _cacheMemory;
    private _blacklisted;
    private _opts;
    constructor(uri: string, folder: string, opts: TSStaticWebSiteOptions);
    getStaticResource(uri: string): [Nullable<Buffer>, string];
    clearCaches(): Promise<void>;
    private _cacheContent;
    private _blacklist;
    private static readonly __TSServerStandardsTypes;
}
export declare type TSParametricQueryDefinition = TSDictionary<TSQueryItem>;
export interface TSParametricEndPoint {
    manager: TSEndPointManager;
    query?: TSParametricQueryDefinition;
}
export declare type TSParametricEndPointsDefinitions = {
    [key in Verb]?: TSParametricEndPoint;
};
export declare class TSParametricEndPoints {
    readonly uri: string;
    readonly depth: number;
    readonly tokens: TSParametricToken[];
    readonly endPoints: TSParametricEndPointsDefinitions;
    readonly caseInsensitive: boolean;
    private _regex;
    private _tlen;
    private static __verbs;
    static validRequestMethod(method: Nullable<string>): Verb | null;
    constructor(path: string, ep: TSEndPoints | TSEndPoint | TSEndPointManager, caseInsensitive?: boolean);
    get static(): boolean;
    execute(req: TSServerRequest, res: ServerResponse): Promise<void>;
    parametersFromPath(path: string): TSParameterDictionary | null;
}
